#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: HostClass.py,v 1.1 2004/01/10 15:13:06 nes Exp $

# FIXME: this file is used only in the parser. It should either be
# renamed or moved elsewhere I think.
# I don't think this class really needs a standalone definition file.

"""
This module contains the class in order to describe a state and
behaviour of Class Host.
"""

import PrintMsg

ExceptionWarn = 'EXEC_WARINING'

class Command:
    """
    Python Class which describe a command. This command can't be execute by a
    class of computers.
    """
    ## ACCESSORS ##
    def GetPriority(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: Accessors of attribut '__Priority'

        @return: the priority of command
        @rtype: integer
	"""
        return int(self.__Priority)

    def GetCommand(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: Accessors of attribut '__Command'

        @return: the name command
        @rtype: string
	"""
        return self.__Command

    def GetLocation(self):
        """
        * Bugs: None

        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: Accessors of attribut '__Location'

        @return: the name of location where the command will be executed
            (for the host or the remote computer)
	"""
        return self.__Location

    ## METHODS ##
    def __cmp__(self, other):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: We override the method to compare

        @return: if 2 objects are the same priority we return 'true'
            else 'false'
	"""
        return -cmp(self.GetPriority(), other.GetPriority())

    ## CONSTRUCTORS ##
    def __init__(self, Priority, Command, Location):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: onstructor

        @param Priority: The number priority of the command
        @param Command: The name of command (could be executed)
        @param Location: The name of location for example the remote or local
        @return: the object which is created
        @rtype: Command instanciation
	"""
        self.__Priority = Priority
        self.__Command 	= Command
        self.__Location	= Location

    ## DEBUG ##
    def __str__(self):
        return "Command: " + self.__Command + ", have priority: " + \
               self.__Priority + ", from " + self.__Location

class VarEnv:
    """
    This class describes the creation or the state of variable environnement
    """
    ## ACCESSEURS ##
    def GetVar(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: Accessors of attribut '__Var'

        @return: the name of variable
	"""
        return self.__Var

    def GetValue(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: Accessors of attribut '__Value'

        @return: the value of variable
	"""
        return self.__Value

    ## METHODS ##
    def __eq__(self, Other):
        """
        Method: return 1 if Other and self are the same variable with
        same values.
	"""
    	return (self.__Var == Other.GetVar()) \
               and (self.__Value == Other.GetValue())



    ## CONSTRUCTEUR ##
    def __init__(self, Var, Value):
        """
        We build an object with set the attributes var and value
	"""
        self.__Var = Var
        self.__Value = Value

    ## DEBUG ##
    def __str__(self):
        return "Variable environnement:" + self.__Var + "=" + self.__Value

class HostCaracter:
    """
    This class describes this host architecture which is caracted by the name
    of architecture, the operating system name and version of last
    """
    ## CONSTRUCTOR ##
    def __init__(self, Archi, Os, Version):
        """
        We build object with set the attributes of one architecture
	"""
        self.__Archi 	= Archi
	self.__Os 	= Os
	self.__Version 	= Version

    def getArchi(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: Accessors of attribut '__Archi'

        @return: the name of architecture
	"""
        return self.__Archi

    def getOs(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: Accessors of attribut '__Os'

        @return: the operating system name
	"""
        return self.__Os

    def getVersion(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: Accessors of attribut '__Version'

        @return: the operating system version
	"""
        return self.__Version

    ## DEBUG ##
    def __str__(self):
        return "Architecture: " + self.__Archi + ", OS: " + self.__Os + \
               ", Version: " + self.__Version


# Classe decrivant une classe de machines
class HostClass:
    """
    This class describes a machin class. This class allows the factorize the
    the machines configuration (we group in the same class, the same characters)
    """
    ## MODIFIERS ##
    def AddExtends(self, Extends):
        """
        * Exceptions raised:
            - ExceptionWarn: in order to indicate multidefinition of attributs
            extends

        * Brief: Modifier of attribut '__Extends' (Mothers machine name)

        We can't add twice else we raise warning exception

        @param Extends: Mothers class to add
	"""
    	if (not self.__Extends):
	    self.__Extends = Extends
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleAttr(parserXMLconf.ATTR_EXTENDS, parserXMLconf.B_CLASS)

    def AddMtdConnec(self, MtdConnec):
        """
        * Exceptions raised:
            - ExceptionWarn: in order to indicate multidefinition of
            method connection object

        * Brief: Modifier of attribut '__ListMtdConnec', add in this list

        We can't add the same object else we raise warning exception

        @param MtdConnec: a method connection to add
        @type MtdConnec: Command
	"""
        if (MtdConnec not in self.__ListMtdConnec):
    	    self.__ListMtdConnec.append(MtdConnec)
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_CMD, parserXMLconf.B_CONNECT_MTD)

    def AddCmdConnec(self, CmdConnec):
        """
        * Exceptions raised:
            - ExceptionWarn in order to indicate multidefinition of
            command connection object

        * Brief: Modifier of attribut '__ListCmdConnec', add in this list

        We can't add the same object else we raise warning exception

        @param CmdConnec: a command connection to add
        @type CmdConnec: Command

	"""
        if (CmdConnec not in self.__ListCmdConnec):
    	    self.__ListCmdConnec.append(CmdConnec)
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_CMD, parserXMLconf.B_CONNECT_CMD)

    def AddCmdUpload(self, CmdUpload):
        """
        * Exceptions raised:
            - ExceptionWarn in order to indicate multidefinition of
            command upload object

        * Brief: Modifier of attribut '__ListCmdUpload', add in this list

        We can't add the same object else we raise warning exception

        @param CmdUpload: a command upload to add
        @type CmdUpload: Command
	"""
        if (CmdUpload not in self.__ListCmdUpload):
    	    self.__ListCmdUpload.append(CmdUpload)
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_CMD, parserXMLconf.B_UPL_CMD)

    def AddMtdUpload(self, MtdUpload):
        """
        * Exceptions raised:
            - ExceptionWarn in order to indicate multidefinition of
            method upload object

        * Brief: Modifier of attribut '__ListMtdUpload', add in this list

        We can't add the same object else we raise warning exception

        @param MtdUpload: a method upload to add
        @type MtdUpload: Command
	"""
        if (MtdUpload not in self.__ListMtdUpload):
    	    self.__ListMtdUpload.append(MtdUpload)
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_CMD, parserXMLconf.B_UPL_MTD)

    def AddVar(self, Var, Value):
        if not self.__DicVar.has_key(Var):
            self.__DicVar[Var] = Value
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_VAR, parserXMLconf.B_VAR_ENV)

    def AddVarEnv(self, VarEnv):
        """
        * Exceptions raised:
            - ExceptionWarn in order to indicate multidefinition of
            environnement variable object

        * Brief: Modifier of attribut '__DicVarEnv', add in this dictionnary

        We can't add the same object else we raise warning exception

        @param VarEnv: a environnement variable to add
        @type VarEnv: VarEnv
	"""
        if (not self.__DicVarEnv.has_key(VarEnv.GetVar())):
            self.__DicVarEnv[VarEnv.GetVar ()] = VarEnv
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_VAR, parserXMLconf.B_VAR_ENV)

    def AddDir(self, Dir):
        """
        * Exceptions raised:
            - ExceptionWarn in order to indicate multidefinition of balise Dir

        * Brief: Modifier of attribut '__Dir' (Working folder)

        We can't add twice else we raise warning exception

        @param Dir: Working folder to add
	"""
        if (not self.__Dir):
    	   self.__Dir = Dir
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_DIR, parserXMLconf.B_CLASS)

##    def AddShell(self, Shell):
##        """
##        * Exceptions raised:
##            - ExceptionWarn in order to indicate multidefinition of balise shell
##
##        * Brief: Modifier of attribut '__Shell' (User shell)
##
##        We can't add twice else we raise warning exception
##
##        @param Shell: Name of user shell to add
##        """
##        if (not self.__Shell):
##       	    self.__Shell = Shell
##	else:
##	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_SH, parserXMLconf.B_CLASS)

    def AddHostCar(self, HostCar):
        """
        * Exceptions raised:
            - ExceptionWarn in order to indicate multidefinition of balise
            Attributs host

        * Brief: Modifier of attribut '__HostCar' (Charaters of one
        architecture)

        We can't add twice else we raise warning exception

        @param HostCar: Attributs host to add
        @type HostCar: HostCar
	"""
        if (not self.__HostCar):
       	    self.__HostCar = HostCar
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_HOST_ATTR, parserXMLconf.B_CLASS)



    def AddMd5Locale(self, Md5Local):
        """
        * Exceptions raised:
            - ExceptionWarn in order to indicate multidefinition of balise
            Md5Local

        * Brief:
        Modifier of attribut '__Md5Local' (Md5 Command executed on local host)

        We can't add twice else we raise warning exception

        @param Md5Local: Md5 Command to add
	"""
        if (not self.__Md5Local):
       	    self.__Md5Local = Md5Local
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_MD5L, parserXMLconf.B_CLASS)


    def AddMd5Remote(self, Md5Remote):
        """
        * Exceptions raised:
            - ExceptionWarn in order to indicate multidefinition of balise
            Md5Remote

        * Brief: Modifier of attribut '__Md5Remote' (Md5 Command executed on
        remote host)

        We can't add twice else we raise warning exception

        @param Md5Remote: Md5 Command to add
	"""
        if (not self.__Md5Remote):
       	    self.__Md5Remote = Md5Remote
	else:
	    raise ExceptionWarn, PrintMsg.GetCodeXMLMultipleDef(parserXMLconf.B_MD5R, parserXMLconf.B_CLASS)


    ## CONSTRUCTOR ##
    def __init__(self, Name):
        self.__Name 		= Name	# Class machine name
        self.__Extends		= ""	# Mothers classes
        self.__ListMtdConnec	= []	# Method connections list
        self.__ListCmdConnec	= []	# Command connections list
        self.__ListCmdUpload	= []	# Command uploads list
	self.__ListMtdUpload	= []	# Method uploads list
        self.__DicVar           = {}	# Environnement variables list
        self.__DicVarEnv 	= {}	# Environnement variables list
        self.__Dir 		= ""	# Working folder
##        self.__Shell		= ""	# User shell
        self.__Md5Local		= ""	# Md5 Commanf executed on local host
        self.__Md5Remote	= ""	# Md5 Command executed on remote host
        self.__HostCar		= None	# Charaters of one architecture

    ## ACCESSORS ##
    def getName(self):
        """
        Accessor of attribut '__Name' (Class machine name)
	"""
        return self.__Name

    def getExtends(self):
        """
        Accessor of attribut '__Extends' (Mothers classes)
	"""
        return self.__Extends

    def getEnv(self):
        """
        Accessor of attribut '__DicVarEnv' (Environnement variables list)
	"""
        return self.__DicVarEnv

    def getVars(self):
        """
        Accessor of attribut '__DicVar' (Variables list)
	"""
        return self.__DicVar

    def getHostCar(self):
        """
        Accessor of attribut '__HostCar' (Charaters of one architecture)
	"""
        return self.__HostCar

    def getTmpDir(self):
        """
        Accessor of attribut '__Dir' (Working folder)
	"""
        return self.__Dir

    def getCmdUpload(self):
        """
        Accessor of attribut '__ListCmdUpload' (Command uploads list)
        """
        self.__ListCmdUpload.sort()
        return self.__ListCmdUpload

    def getCmdConnec(self):
        """
        Accessor of attribut '__ListCmdConnec' (Command connections list)
	"""
        self.__ListCmdConnec.sort()
        return self.__ListCmdConnec

    def getMd5Local(self):
        """
        Accessor of attribut '__Md5Local' (Md5 Cmd executed on local host)
	"""
        return self.__Md5Local

    def getMd5Remote(self):
        """
        Accessor of attribut '__Md5Remote' (Md5 Cmd executed on remote host)
	"""
        return self.__Md5Remote

    ## DEBUG ##
    def __str__(self):
	StrListMtdConnec	= "  List of methods connection:\n"
	StrListCmdConnec	= "  List of commands connection:\n"
	StrListCmdUpload	= "  List of command upload:\n"
	StrListMtdUpload	= "  List of method upload:\n"
	StrDicVarEnv		= "  List of environnement variable:\n"

        for i in self.__ListMtdConnec:
	    StrListMtdConnec += "    " + str(i) + "\n"
        for i in self.__ListCmdConnec:
	    StrListCmdConnec += "    " + str(i) + "\n"
        for i in self.__ListCmdUpload:
	    StrListCmdUpload += "    " + str(i) + "\n"
        for i in self.__ListMtdUpload:
	    StrListMtdUpload += "    " + str(i) + "\n"
        for i in self.__DicVarEnv:
	    StrDicVarEnv += "    " + i + "=" + self.__DicVarEnv[i].GetValue() + "\n"
##	if (self.__HostCar != None):
##            return "Host Class: " + self.__Name + ", extends: " + self.__Extends + "\n" + StrListMtdConnec + StrListCmdConnec + StrListCmdUpload + StrListMtdUpload + StrDicVarEnv + "\nDir: " + self.__Dir + "\nShell: " + self.__Shell + "\n" +self.__HostCar.__str__()
##        else:
##            return "Host Class: " + self.__Name + ", extends: " + self.__Extends + "\n" + StrListMtdConnec + StrListCmdConnec + StrListCmdUpload + StrListMtdUpload + StrDicVarEnv + "\nDir: " + self.__Dir + "\nShell: " + self.__Shell + "\n"
        if (self.__HostCar != None):
            return "Host Class: " + self.__Name + ", extends: " + self.__Extends + "\n" + StrListMtdConnec + StrListCmdConnec + StrListCmdUpload + StrListMtdUpload + StrDicVarEnv + "\nDir: " + self.__Dir + "\n" +self.__HostCar.__str__()
        else:
            return "Host Class: " + self.__Name + ", extends: " + self.__Extends + "\n" + StrListMtdConnec + StrListCmdConnec + StrListCmdUpload + StrListMtdUpload + StrDicVarEnv + "\nDir: " + self.__Dir + "\n"
