#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
Parse a XML file which describe a compile farm. Also
we created vampire object associed to the description of this compile
farm.
"""

import sys
from xml.parsers import expat

import PrintMsg
import HostList
import HostClass
import Location
import InfoBalise

  #====------------====#
 #  STATICS ELEMENTS  #
#====------------====#

# Balise
B_ROOT="hosts_config"
B_CLASS="class"
B_CONNECT_MTD="connection_method"
B_CONNECT_CMD="connection_command"
B_UPL_CMD="upload_command"
B_UPL_MTD="upload_method"
B_CMD="command"
B_DIR="tmpdir"
B_ENV="env"
B_VAR="var"
##B_SH="shell"
B_NAME="name"
B_HOST_ATTR="host_car"
B_HOST_LIST="hostlist"
B_DOC="docstring"
B_MD5L="md5cmd_local"
B_MD5R="md5cmd_remote"

# Attributs
ATTR_NAME="name"
ATTR_VALUE="value"
ATTR_PRIORITY="priority"
ATTR_EXTENDS="extends"
ATTR_OS="os"
ATTR_ARCH="archi"
ATTR_VERSION="version"
ATTR_CLASS="class"
ATTR_LOC="loc"
ATTR_CMT="comment"
ATTR_HEADER="xmlns:xsi"

# Defaults values
DFT_LOC="local"

"""
This class is used to validate a XML file follows the XMLSchema
(doc/config/hosts.xsd)
"""
class HostConfValidator:
    ## ACCESSORS ##
    def GetError(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: To get message error

        @return: data message
        """
        return self.__Error

    def GetErrorLine(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: To get Line where the error was raised

        @return: error line
        @rtype: integer
        """
        return self.__ErrorLine

    def GetErrorColumn(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: To get colmun where the error was raised
        @return: error colmun
        @rtype: integer
        """
        return self.__ErrorCol

    ## MODIFIERS ##
    def ResetError(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: This methods deactivates the error state of validator
        """
        self.__Error = ""
	self.__ErrorLine = -1
	self.__ErrorCol	 = -1

    ## METHODS ##
    def RaiseExpat(self, Error, Name, i):
        """
        * Exceptions raised:
            - expat.ExpatError

        * Exceptions intercepted: None

        * Brief: Methods in order to raise expat exception detected by this
        validator

        @param Error: Data of error message
        @type Error: string
        @param Name: Balise name where was raised exception
        @type Name: string
        @param i: i th balise where was raised exception
        @type i: string
	"""
    	self.__Error = Error
    	Result = Location.GetLocationBegin(self.__File, Name, i)
    	self.__ErrorLine = Result[0]
    	self.__ErrorCol  = Result[1]
    	raise expat.ExpatError

    def __GetIbal(self, Name):
        """
        * Brief: Set the i th balise, this balise has just read

        @param Name: Current balise name
	"""
        if (Name == B_ROOT):
            return self.NbRoot
	elif (B_CLASS == Name):
            return self.NbClass
	elif (B_CONNECT_MTD == Name):
            return self.NbConMtd
	elif (B_CONNECT_CMD == Name):
            return self.NbConCmd
	elif (B_UPL_CMD == Name):
            return self.NbUplCmd
	elif (B_UPL_MTD == Name):
            return self.NbUplMtd
	elif (B_CMD == Name):
            return self.NbCmd
	elif (B_DIR == Name):
            return self.NbDir
	elif (B_ENV == Name):
            return self.NbEnv
	elif (B_VAR == Name):
            return self.NbVar
##	elif (B_SH == Name):
##            return self.NbShell
	elif (B_NAME == Name):
            return self.NbName
	elif (B_HOST_ATTR == Name):
            return self.NbHostAtr
	elif (B_HOST_LIST == Name):
            return self.NbHostLst
	elif (B_MD5L == Name):
            return self.NbMd5l
	elif (B_MD5R == Name):
            return self.NbMd5r
	elif (B_DOC != Name):
            return 1
	return 0

    def Enter(self, Name, attrs):
        """
        * Exceptions raised:
            - expat.ExpatError: was raised when intercepts any other
            exception. Also we raise it when there
            is unknown balise, unknown attributs and
            lake of attributs.

        * Exceptions intercepted:
            - KeyError: When we use attributs dictionnary, we check the
            necessary attributs in balise.
            - ValueError: When we use a convertion a string to integer, so
            we check the typage of attribut.

        * Brief: This method is called when the parser begins with balise 'Name'

        @param Name: Entering balise name
        @type Name: string
        @param attrs: Attributs (with values) in balise
        @type attrs: Dictionnary of string
	"""    
	try:
	    Father = self.__TreeBal.pop()
        except IndexError:
	    Father = ""
	self.__TreeBal.append(Father)
        self.__TreeBal.append(Name)
	i = 0
        if (Name == B_ROOT):
	    self.NbRoot += 1
	    i = self.NbRoot
	elif (B_CLASS == Name):
	    self.NbClass += 1
	    i = self.NbClass
	elif (B_CONNECT_MTD == Name):
	    self.NbConMtd += 1
	    i = self.NbConMtd
	elif (B_CONNECT_CMD == Name):
	    self.NbConCmd += 1
	    i = self.NbConCmd
	elif (B_UPL_CMD == Name):
	    self.NbUplCmd += 1
	    i = self.NbUplCmd
	elif (B_UPL_MTD == Name):
	    self.NbUplMtd += 1
	    i = self.NbUplMtd
	elif (B_CMD == Name):
	   self.NbCmd += 1
	   i = self.NbCmd
	   try:
	       try:
                   x = int(attrs[ATTR_PRIORITY])
	       except KeyError:
                   self.__Error = PrintMsg.GetCodeXMLExceptAttr(ATTR_PRIORITY, B_CMD)
                   Result = Location.GetLocationBegin(self.__File, Name, self.NbCmd)
                   self.__ErrorLine = Result[0]
                   self.__ErrorCol  = Result[1]
                   raise expat.ExpatError
           except ValueError :
	       self.__Error = PrintMsg.GetCodeXMLTypage("Integer", attrs[ATTR_PRIORITY])
	       Result = Location.GetLocationBegin(self.__File, Name, self.NbCmd)
	       self.__ErrorLine = Result[0]
	       self.__ErrorCol  = Result[1]
	       raise expat.ExpatError
	elif (B_DIR == Name):
	    self.NbDir += 1
	    i = self.NbDir
	elif (B_ENV == Name):
	    self.NbEnv += 1
	    i = self.NbEnv
	elif (B_VAR == Name):
	    self.NbVar += 1
	    i = self.NbVar
##	elif (B_SH == Name):
##	    self.NbShell += 1
##	    i = self.NbShell
	elif (B_NAME == Name):
	    self.NbName += 1
	    i = self.NbName
	elif (B_HOST_ATTR == Name):
	    self.NbHostAtr += 1
	    i = self.NbHostAtr
	elif (B_HOST_LIST == Name):
	    self.NbHostLst += 1
	    i = self.NbHostLst
	elif (B_MD5L == Name):
	    self.NbMd5l += 1
	    i = self.NbMd5l
	elif (B_MD5R == Name):
	    self.NbMd5r += 1
	    i = self.NbMd5r
	elif (B_DOC != Name):
	    self.__Error = PrintMsg.GetCodeXMLUnknowBal(Name)
	    Result = Location.GetLocationBegin(self.__File, Name, 1)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError
	Ret = self.__InfoBal.IsRightAttr(Name, attrs)
        if (Ret != ""):
	    if (Ret[:1] == "R"):
	        self.__Error = PrintMsg.GetCodeXMLExceptAttr(Ret[1:], Name)
	    else:
	        self.__Error = PrintMsg.GetCodeXMLUnexcpAttr(Ret[1:], Name)
	    Result = Location.GetLocationBegin(self.__File, Name, i)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError
	if (not self.__InfoBal.IsRightBalise(Father, Name)):
	    self.__Error = PrintMsg.GetCodeXMLUnexcpBalBis(Father, Name)
	    Result = Location.GetLocationBegin(self.__File, Name, i)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError

    def Exit(self, Name, Data):
        """
        * Exceptions raised:
            - expat.ExpatError: we raise it when there is unknown
            balise, lake of balise.

        * Exceptions intercepted: None

        * Brief: This method is called when the parser finishes with balise
        'Name'

        @param Name: Exiting balise name
        @type Name: string
        @param Data: Data between entering balise and exiting balise
        @type Data: string
	"""
        i = self.__GetIbal(Name)
	if (not self.__InfoBal.GetInfoBal().has_key(Name)):
	    self.__Error = PrintMsg.GetCodeXMLUnknowBal(Name)
	    Result = Location.GetLocationEnd(self.__File, Name, 1)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError
        try:
	    self.__TreeBal.pop()
	except IndexError:
	    self.__Error = PrintMsg.GetCodeXMLUnexcpBal(Name)
	    Result = Location.GetLocationEnd(self.__File, Name, i)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError

    def ReceiveData(self, Name, Data):
        """
        * Exceptions raised:
            - expat.ExpatError: we raise it when there is unexcepted
            data between balises.

        * Exceptions intercepted: None

        * Brief: This method is called when the parser finishes with data of
        balise 'Name'

        @param Name: Current balise name
        @type Name: string
        @param Data: Data between entering balise and exiting balise
	"""
        i = self.__GetIbal(Name)
        if (not self.__InfoBal.IsRightData(Name, Data)):
	    self.__Error = PrintMsg.GetCodeXMLUnexcpData(Name, Data)
	    Result = Location.GetLocationBegin(self.__File, Name, i)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError

    ## CONSTRUCTOR ##
    def __init__(self, Parser, File):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: his method builds the validator (i th balise of all balises,
        info balise depends of XML schema and error state)

        @param Parser: A reference on expat parser
        @type Parser: expat object
        @param File: Name of parsing file (in order to format errors messages)
	"""
        self.__Parser	= Parser
        self.__File	= File
	self.__TreeBal	= []
        self.__Error	= ""
        self.__ErrorLine= -1
        self.__ErrorCol	= -1
        self.NbRoot 	= 0
        self.NbClass	= 0
        self.NbConMtd	= 0
        self.NbConCmd	= 0
        self.NbUplMtd	= 0
        self.NbUplCmd	= 0
        self.NbCmd	= 0
        self.NbDir	= 0
        self.NbEnv	= 0
        self.NbVarEnv	= 0
        self.NbVar	= 0
##        self.NbShell	= 0
        self.NbName	= 0
        self.NbHostAtr	= 0
        self.NbHostLst	= 0
	self.NbMd5l	= 0
	self.NbMd5r	= 0
        self.__Name     = 0
	self.__InfoBal	= InfoBalise.InfoBalise(B_ROOT, {
  	    B_ROOT	 : [	[],		# Liste attr obligatoires
	    			[ATTR_HEADER],	# Liste attr optionnels
				[B_HOST_LIST],  # Liste balises obligatoires
				[B_CLASS],	# Liste balises optionnelles
				0],		# 1=Si accepte des donnees, 0=sinon
  	    B_CLASS	 : [	[ATTR_NAME],
	    			[ATTR_EXTENDS],
				[],
##				[B_MD5L, B_MD5R, B_HOST_ATTR, B_CONNECT_MTD, B_CONNECT_CMD, B_UPL_MTD, B_UPL_CMD, B_DIR, B_ENV, B_SH],
                                [B_MD5L, B_MD5R, B_HOST_ATTR, B_CONNECT_MTD, B_CONNECT_CMD, B_UPL_MTD, B_UPL_CMD, B_DIR, B_ENV, B_VAR],
				0],
  	    B_CONNECT_MTD: [	[],
	    			[],
				[],
				[B_CMD],
				0],
  	    B_CONNECT_CMD: [	[],
	    			[],
				[],
				[B_CMD],
				0],
  	    B_UPL_CMD	 : [	[],
	    			[],
				[],
				[B_CMD],
				0],
  	    B_UPL_MTD	 : [	[],
	    			[],
				[],
				[B_CMD],
				0],
  	    B_CMD	 : [	[ATTR_PRIORITY],
	    			[ATTR_LOC],
				[],
				[],
				1],
  	    B_DIR	 : [	[],
	    			[],
				[],
				[],
				1],
  	    B_ENV	 : [	[],
	    			[],
				[],
				[B_VAR],
				0],
            B_VAR	 : [	[ATTR_NAME, ATTR_VALUE],
	    			[ATTR_CMT],
				[],
				[],
				0],
##  	    B_SH	 : [	[],
##	    			[],
##				[],
##				[],
##				1],
  	    B_NAME	 : [	[],
	    			[],
				[],
				[],
				1],
  	    B_HOST_ATTR	 : [	[ATTR_ARCH, ATTR_OS, ATTR_VERSION],
	    			[],
				[],
				[],
				0],
  	    B_HOST_LIST	 : [	[ATTR_CLASS],
	    			[],
				[],
				[B_NAME],
				0],
  	    B_MD5L	 : [	[],
	    			[],
				[],
				[],
				1],
	    B_MD5R	 : [	[],
	    			[],
				[],
				[],
				1],
	    B_DOC	 : [	[],
	    			[],
				[],
				[],
				1],
	})

class HostConfParser:
    ## ACCESSEURS ##
    def GetListClass(self):
        return self.__ListClass

    def GetListHost(self):
        return self.__ListHost

    ## CODE PARSER XML ##
    def StartElement(self, name, attrs):
        """
        * Exceptions raised:
            - expat.ExpatError: Automatic raised by expat when there is
            syntax error.

        * Brief: Prototype expat, when it enters in bal

        @param name: Entering balise name
        @type name: string
        @param attrs: Attributs (with values) in balise
        @type attrs: Dictionnary of stringise 'name' with attributs 'attrs'
	"""
	if (name == B_ENV):
	    self.__InEnv = 1
        elif (name == B_CONNECT_CMD):
	    self.__ConnectCmd = 1
        elif (name == B_CONNECT_MTD):
	    self.__ConnectMtd = 1
	elif (name == B_UPL_CMD):
	    self.__UplCommand = 1
	elif (name == B_UPL_MTD):
	    self.__UplMethod = 1
        elif (name == B_CLASS):
	    if (attrs.has_key(ATTR_NAME)):
	        self.__CurClass = HostClass.HostClass(attrs[ATTR_NAME])
	    if ((len(attrs) == 2) and attrs.has_key(ATTR_EXTENDS)):
	        self.__CurClass.AddExtends(attrs[ATTR_EXTENDS])
        elif (name == B_HOST_LIST):
	    if (attrs.has_key(ATTR_CLASS)):
	        self.__CurHostList = HostList.HostList(attrs[ATTR_CLASS])
        # Initialisation des attributs d'une balises
	self.__Data = ""
        self.__Balise.append(name)
	self.__Attr.append(attrs)
	self.__Validator.Enter(name, attrs)

    # Action a faire pour les donnees d'une balise
    def CharacterData(self, data):
        """
        * Exceptions raised:
            - expat.ExpatError: Automatic raised by expat when there is
            syntax error.

        * Brief: Prototype expat, when it receives data in balise

        @param data: Data in between two balises
        @type data: string
	"""
        try:
	    name = self.__Balise.pop()
	except IndexError:
	    pass
	self.__Balise.append(name)
	self.__Data += data
	self.__Validator.ReceiveData(name, data)

    # Action a faire a la fermeture d'une balise
    def EndElement(self, name):
        """
        * Exceptions raised:
            - expat.ExpatError: Automatic raised by expat when there is
            syntax error.

        * Exceptions intercepted:
            - HostClass.ExceptionWarn: when there is multiple defition or on
            generally when there is too attributs for one characters or when
            there is a lake of HotClass characters.

        * Brief: Prototype expat, when it exits in balise 'name'

        @param name: Exiting balise name
        @type name: string
	"""
        try:
	    self.__Balise.pop()
	except IndexError:
            pass
        self.__Validator.Exit(name, self.__Data)
	TmpAttr = self.__Attr.pop()
        if (name == B_ROOT):
	    if (self.__ListHost != []):
                PrintMsg.SuccessXML(self.__File)
	    else:
                self.__Validator.RaiseExpat(PrintMsg.GetCodeXMLExceptBal(B_HOST_LIST, B_ROOT), B_ROOT, 1)
        elif (name == B_CLASS):
            if (self.__CurClass):
                self.__ListClass.append(self.__CurClass)
                self.__CurClass = None
        elif (name == B_CMD):
	    if (TmpAttr.has_key(ATTR_LOC)):
	        Loc = TmpAttr[ATTR_LOC]
	    else:
	        Loc = DFT_LOC
            if (TmpAttr.has_key(ATTR_PRIORITY)):
	        try:
	            if ((self.__CurClass) and (self.__UplCommand == 1)):
	                self.__CurClass.AddCmdUpload(HostClass.Command(TmpAttr[ATTR_PRIORITY], self.__Data, Loc))
                    if ((self.__CurClass) and (self.__ConnectCmd == 1)):
                        self.__CurClass.AddCmdConnec(HostClass.Command(TmpAttr[ATTR_PRIORITY], self.__Data, Loc))
	            if ((self.__CurClass) and (self.__ConnectMtd == 1)):
		        self.__CurClass.AddMtdConnec(HostClass.Command(TmpAttr[ATTR_PRIORITY], self.__Data, Loc))
	            if ((self.__CurClass) and (self.__UplMethod == 1)):
                        self.__CurClass.AddMtdUpload(HostClass.Command(TmpAttr[ATTR_PRIORITY], self.__Data, Loc))
	        except HostClass.ExceptionWarn, code:
		    Res = Location.GetLocationBegin(self.__File, B_CMD, self.__Validator.NbCmd)
		    print PrintMsg.WarningXML(self.__File, Res[0], Res[1], code)
	elif (name == B_CONNECT_MTD):
            self.__ConnectMtd = 0
	elif (name == B_CONNECT_CMD):
            self.__ConnectCmd = 0
	elif (name == B_UPL_CMD):
	    self.__UplCommand = 0
	elif (name == B_UPL_MTD):
            self.__UplMethod = 0
	elif (name == B_DIR):
            if (self.__CurClass):
	        try:
	            self.__CurClass.AddDir(self.__Data);
	        except HostClass.ExceptionWarn, code:
  		    Res = Location.GetLocationBegin(self.__File, B_DIR, self.__Validator.NbDir)
		    print PrintMsg.WarningXML(self.__File, Res[0], Res[1], code)

	elif (name == B_ENV):
            self.__InEnv = 0
        elif (name == B_VAR):
            if self.__CurClass and TmpAttr.has_key(ATTR_NAME) and TmpAttr.has_key(ATTR_VALUE):
                if self.__InEnv == 1:
		    try:
		        self.__CurClass.AddVarEnv(HostClass.VarEnv(TmpAttr[ATTR_NAME], TmpAttr[ATTR_VALUE]))
	            except HostClass.ExceptionWarn, code:
  		        Res = Location.GetLocationBegin(self.__File, B_VAR, self.__Validator.NbVar)
		        print PrintMsg.WarningXML(self.__File, Res[0], Res[1], code)
                else:
                    try:
                        self.__CurClass.AddVar(TmpAttr[ATTR_NAME], TmpAttr[ATTR_VALUE])
                    except HostClass.ExceptionWarn, code:
                        Res = Location.GetLocationBegin(self.__File, B_VAR, self.__Validator.NbVar)
                        print PrintMsg.WarningXML(self.__File, Res[0], Res[1], code)

##        elif (name == B_SH):
##            if (self.__CurClass):
##	        try:
##                   self.__CurClass.AddShell(self.__Data);
##	        except HostClass.ExceptionWarn, code:
##		    Res = Location.GetLocationBegin(self.__File, B_SH, self.__Validator.NbShell)
##		    print PrintMsg.WarningXML(self.__File, Res[0], Res[1], code)

	elif (name == B_NAME):
	    if (self.__CurHostList):
	        self.__CurHostList.AddName(self.__Data)
	elif (B_MD5L == name):
	    if (self.__CurClass):
	        self.__CurClass.AddMd5Locale(self.__Data)
	elif (B_MD5R == name):
	    if (self.__CurClass):
	        self.__CurClass.AddMd5Remote(self.__Data)
	elif (name == B_HOST_ATTR):
            if (self.__CurClass):
                if (TmpAttr.has_key(ATTR_ARCH) and TmpAttr.has_key(ATTR_OS) and TmpAttr.has_key(ATTR_VERSION)):
                    try:
		        self.__CurClass.AddHostCar(HostClass.HostCaracter(TmpAttr[ATTR_ARCH], TmpAttr[ATTR_OS], TmpAttr[ATTR_VERSION]))
	            except HostClass.ExceptionWarn, code:
		        Res = Location.GetLocationBegin(self.__File, B_HOST_ATTR, self.__Validator.NbHostAtr)
		        print PrintMsg.WarningXML(self.__File, Res[0], Res[1], code)
	elif (name == B_HOST_LIST):
	    if (self.__CurHostList):
                self.__ListHost.append(self.__CurHostList)
                self.__CurHostList = None

    ## CONSTRUCTOR ##
    def __init__(self, File):
        """
        * Exceptions raised: None

        * Exceptions intercepted:
            - expat.ExpatError: We intercepted all exceptions raised
            by others object or others methos of this object.
            - IOError: When we have invalide file 'File'

        * Brief: The parser starting, we connect the expat parser to own parser
        and validator for XML Shema (doc/config/hosts.xsd)

        @param File: File name to parse
        @type File: string
	"""
        self.__File		= File
        self.__InEnv 		= 0
        self.__ConnectMtd 	= 0
        self.__UplCommand 	= 0
        self.__UplMethod	= 0
	self.__ConnectCmd	= 0
        self.__ListClass 	= []
        self.__ListHostList 	= []
	self.__ListHost		= []
        self.__CurClass 	= None
        self.__CurHostList	= None
        self.__Balise 		= []
        self.__Attr 		= []
        self.__Data 		= ""
        self.__Parser = expat.ParserCreate()
        self.__Parser.StartElementHandler	= self.StartElement
        self.__Parser.EndElementHandler		= self.EndElement
        self.__Parser.CharacterDataHandler	= self.CharacterData
	try:
	    self.__Validator = HostConfValidator(self.__Parser, self.__File)
            try:
	        self.__Parser.ParseFile(open(self.__File))
   	    except expat.ExpatError:
	        if (self.__Validator.GetError() == ""):
		    PrintMsg.ErrorXML(self.__File, self.__Parser.ErrorLineNumber, self.__Parser.ErrorColumnNumber, expat.ErrorString(self.__Parser.ErrorCode))
	        else:
		    PrintMsg.ErrorXML(self.__File, self.__Validator.GetErrorLine(), self.__Validator.GetErrorColumn(), self.__Validator.GetError())
	        self.__Validator.ResetError()
	except IOError, (errno, strerror):
            PrintMsg.VError(self.__File, strerror)

    ## DEBUG ##
    def __str__(self):
        Res = "StateParserConf:\n"

	for i in self.__ListClass:
	        Res = Res + str(i) + "\n"
	for i in self.__ListHost:
	        Res = Res + str(i) + "\n"
        return Res

if __name__ == "__main__":
    """
    ##Debug## used by test/check-vampire.py
    """
    i = ParserConf(sys.argv[1])
    print str(i)
