#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This module could parse a XML file which describe a tarballs pool. Also
we created vampire object associed to the description of this pool.
"""

import sys
from xml.parsers import expat

import TarballClass
import Tarball
from core import TarballCommand
import PrintMsg
import Location
import InfoBalise

  #====------------====#
 # ELEMENTS STATIQUES #
#====------------====#

# Les balises possibles
B_ROOT="tarballs_config"
B_CLASS="class"
B_ONFAIL="onfail"
B_ONSUCCESS="onsuccess"
B_BUILD_CMD="build_commands"
B_DOC="docstring"
B_VAR="var"
B_CMD="command"
B_TEST_OS="testable_host"

# Les attributs possibles
ATTR_NAME="name"
ATTR_CMD="command"
ATTR_ARG="arg"
ATTR_CLASS="class"
ATTR_EXTENDS="extends"
ATTR_TYPE="type"
ATTR_VALUE="value"
ATTR_HEADER="xmlns:xsi"
ATTR_RESTRICTION="restriction"

# Les valeurs par defaut

class TarballConfValidator:
    ## ACCESSORS ##
    def GetError(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: To get message error

        @return: data message
        """
        return self.__Error

    def GetErrorLine(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: To get Line where the error was raised

        @return: error line
        @rtype: integer
        """
        return self.__ErrorLine

    def GetErrorColumn(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: To get colmun where the error was raised

        @return: error colmun
        @rtype: integer
        """
        return self.__ErrorCol

    ## MODIFIERS ##
    def ResetError(self):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: This methods deactivates the error state of validator
	"""
        self.__Error = ""
	self.__ErrorLine = -1
	self.__ErrorCol	 = -1

    ## METHODS ##
    def RaiseExpat(self, Error, Name, i):
        """
        * Exceptions raised:
            - expat.ExpatError

        * Exceptions intercepted: None

        * Brief: Methods in order to raise expat exception detected by this
        validator

        @param Error: Data of error message
        @type Error: string
        @param Name: Balise name where was raised exception
        @type Name: string
        @param i: i th balise where was raised exception
        @type i: string
	"""
    	self.__Error = Error
    	Result = Location.GetLocationBegin(self.__File, Name, i)
    	self.__ErrorLine = Result[0]
    	self.__ErrorCol  = Result[1]
    	raise expat.ExpatError

    ## check when the parser enter into balise "name"
    def __GetIbal(self, Name):
        """
        * Brief: Set the i th balise, this balise has just read

        @param Name: Current balise named
	"""
        if (Name == B_ROOT):
            return self.NbRoot
	elif (B_CLASS == Name):
            return self.NbClass
	elif (B_ONFAIL == Name):
            return self.NbOnFail
	elif (B_ONSUCCESS == Name):
            return self.NbOnsucc
	elif (B_BUILD_CMD == Name):
            return self.NbBuildCmd
	elif (B_CMD == Name):
            return self.NbCmd
	elif (B_VAR == Name):
            return self.NbVar
	elif (B_TEST_OS == Name):
            return self.NbTestOs
	elif (B_DOC != Name):
            return 1
	return 0

    def Enter(self, Name, attrs):
        """
        * Exceptions raised:
            - expat.ExpatError: was raised when intercepts any other exception.
            Also we raise it when there is unknown balise, unknown attributs
            and lake of attributs.

        * Exceptions intercepted:
            - KeyError: When we use attributs dictionnary, we check the
            necessary attributs in balise.
            - ValueError: When we use a convertion a string to integer, so
            we check the typage of attribut.

        * Brief: This method is called when the parser begins with balise
        'Name'

        @param Name: Entering balise name
        @type Name: string
        @param attrs: Attributs (with values) in balise
        @type attrs: Dictionnary of string
	"""
	try:
	    Father = self.__TreeBal.pop()
        except IndexError:
	    Father = ""
	self.__TreeBal.append(Father)
        self.__TreeBal.append(Name)
	i = 0
        if (Name == B_ROOT):
	    self.NbRoot += 1
            i = self.NbRoot
	elif (B_CLASS == Name):
	    self.__IsInClass = 1
	    self.NbClass += 1
            i = self.NbClass
	    self.__InClass = {B_BUILD_CMD: 0}
	elif (B_ONFAIL == Name):
	    self.NbOnFail += 1
            i = self.NbOnFail
	elif (B_ONSUCCESS == Name):
	    self.NbOnsucc += 1
            i = self.NbOnsucc
	elif (B_BUILD_CMD == Name):
	    self.NbBuildCmd += 1
            i = self.NbBuildCmd
	    self.__InBCmd = {B_CMD : 0}
            self.__InClass[B_BUILD_CMD] += 1
	elif (B_CMD == Name):
	    self.NbCmd += 1
            i = self.NbCmd
	    self.__InBCmd[B_CMD] += 1
	elif (B_VAR == Name):
	    self.NbVar += 1
            i = self.NbVar
	elif (B_TEST_OS == Name):
	    self.NbTestOs += 1
            i = self.NbTestOs
	elif (B_DOC != Name):
	    self.__Error = PrintMsg.GetCodeXMLUnknowBal(Name)
	    Result = Location.GetLocationBegin(self.__File, Name, 1)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError
	Ret = self.__InfoBal.IsRightAttr(Name, attrs)
        if (Ret != ""):
	    if (Ret[:1] == "R"):
	        self.__Error = PrintMsg.GetCodeXMLExceptAttr(Ret[1:], Name)
	    else:
	        self.__Error = PrintMsg.GetCodeXMLUnexcpAttr(Ret[1:], Name)
	    Result = Location.GetLocationBegin(self.__File, Name, i)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError
	if (not self.__InfoBal.IsRightBalise(Father, Name)):
	    self.__Error = PrintMsg.GetCodeXMLUnexcpBalBis(Father, Name)
	    Result = Location.GetLocationBegin(self.__File, Name, i)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError

    ## check when the parser go out balise "name"
    def Exit(self, Name, Data):
        """
        * Exceptions raised:
           - expat.ExpatError: we raise it when there is unknown
           balise, lake of balise.

        * Exceptions intercepted: None

        * Brief: This method is called when the parser finish

        @param Name: Exiting balise name
        @type Name: string
        @param Data: Data between entering balise and exiting balise
        @type Data: stringes with balise 'Name'
	"""
        i = self.__GetIbal(Name)
	if (B_CLASS == Name):
	    self.__IsInClass = 0
	    if (1 != self.__InClass[B_BUILD_CMD]):
                # FIXME: msg error
		self.__Error = PrintMsg.GetCodeXMLExceptBalBis(Name, self.__InClass[B_BUILD_CMD], "one")
                Result = Location.GetLocationEnd(self.__File, Name, i = self.NbClass)
                self.__ErrorLine = Result[0]
                self.__ErrorCol  = Result[1]
                raise expat.ExpatError
	if (B_BUILD_CMD == Name):
	    if (1 > self.__InBCmd[B_CMD]):
                # FIXME: msg error
		self.__Error = PrintMsg.GetCodeXMLExceptBalBis(Name, self.__InBCmd[B_CMD], "at least one")
                Result = Location.GetLocationEnd(self.__File, Name, self.NbBuildCmd)
                self.__ErrorLine = Result[0]
                self.__ErrorCol  = Result[1]
                raise expat.ExpatError
	if (not self.__InfoBal.GetInfoBal().has_key(Name)):
	    self.__Error = PrintMsg.GetCodeXMLUnknowBal(Name)
	    Result = Location.GetLocationEnd(self.__File, Name, 1)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError
        try:
	    self.__TreeBal.pop()
	except IndexError:
	    self.__Error = PrintMsg.GetCodeXMLUnexcpBal(Name)
	    Result = Location.GetLocationEnd(self.__File, Name, i)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError

    ## Check data
    def ReceiveData(self, Name, Data):
        """
        * Exceptions raised:
           - expat.ExpatError: we raise it when there is unexcepted
           data between balises.

        * Exceptions intercepted: None

        * Brief: This method is called when the parser finishes with data of
        balise 'Name'

        @param Name: Current balise name
        @type Name: string
        @param Data: Data between entering balise and exiting balise
	"""
        i = self.__GetIbal(Name)
        if (not self.__InfoBal.IsRightData(Name, Data)):
	    self.__Error = PrintMsg.GetCodeXMLUnexcpData(Name, Data)
	    Result = Location.GetLocationBegin(self.__File, Name, i)
	    self.__ErrorLine = Result[0]
	    self.__ErrorCol  = Result[1]
	    raise expat.ExpatError


    ## CONSTRUCTEUR ##
    def __init__(self, Parser, File):
        """
        * Exceptions raised: None

        * Exceptions intercepted: None

        * Brief: This method builds the validator (i th balise of all balises,
        info balise depends of XML schema and error state)

        @param Parser: A reference on expat parser
        @type Parser: expat object
        @param File: Name of parsing file (in order to format errors messages)
	"""
        self.__Parser	= Parser
        self.__File	= File
	self.__TreeBal	= []
        self.__Error	= ""
        self.__ErrorLine= -1
        self.__ErrorCol	= -1
        self.NbRoot 	= 0
        self.NbClass	= 0
        self.NbBuildCmd	= 0
        self.NbCmd	= 0
        self.NbOnsucc	= 0
        self.NbOnFail	= 0
        self.NbTestOs	= 0
        self.NbVar	= 0
	self.__InBCmd	= {B_CMD : 0}
        self.__InClass	= {B_BUILD_CMD: 0}
	self___IsInClass= 0
	self.__Name     = 0
	self.__InfoBal	= InfoBalise.InfoBalise(B_ROOT, {
  	    B_ROOT	 : [	[],		# Liste attr obligatoires
	    			[ATTR_HEADER],	# Liste attr optionnels
				[],	# Liste balises obligatoires
				[B_CLASS], # Liste balises optionnelles
				0],		# 1=Si accepte des donnees, 0=sinon
  	    B_CLASS	 : [	[ATTR_NAME],
	    			[ATTR_EXTENDS],
				[B_BUILD_CMD],
				[B_VAR],
				0],
  	    B_BUILD_CMD: [	[],
	    			[],
				[B_CMD],
				[],
				0],
  	    B_CMD:	[	[ATTR_NAME],
	    			[ATTR_TYPE],
				[B_ONFAIL, B_ONSUCCESS],
				[],
				0],
  	    B_ONFAIL	 : [	[ATTR_CMD],
	    			[ATTR_ARG],
				[],
				[],
				0],
  	    B_ONSUCCESS	 : [	[ATTR_CMD],
	    			[ATTR_ARG],
				[],
				[],
				0],
  	    B_TEST_OS	 : [	[],
	    			[],
				[],
				[],
				1],
  	    B_VAR	 : [	[ATTR_NAME, ATTR_VALUE],
	    			[ATTR_RESTRICTION],
				[],
				[],
				0],
  	    B_DOC	 : [	[],
	    			[],
				[],
				[],
				1],
	})

class TarballConfParser:
    ## ACCESSEURS ##
    def GetListClass(self):
        return self.__ListClass

    def GetListTarball(self):
        return self.__ListTarball

    ## CODE PARSER XML ##
    # Action a faire a l'ouverture d'une balise
    def StartElement(self, name, attrs):
        """
        * Exceptions raised:
            - expat.ExpatError	Automatic raised by expat when there is
            syntax error.

        * Brief: Prototype expat, when it enters in balise 'name' with
        attributs 'attrs'

        @param name: Entering balise name
        @type name: string
        @param attrs: Attributs (with values) in balise
        @type attrs: Dictionnary of string
	"""
	self.__Data = ""
        self.__Balise.append(name)
	self.__Attr.append(attrs)
	self.__Validator.Enter(name, attrs)
	if (name == B_CLASS):
	        if (attrs.has_key(ATTR_EXTENDS)):
		    self.__CurClass = TarballClass.TarballClass(attrs[ATTR_NAME], attrs[ATTR_EXTENDS])
		else:
		    self.__CurClass = TarballClass.TarballClass(attrs[ATTR_NAME])
	elif (name == B_CMD):
	    if (attrs.has_key(ATTR_TYPE)):
	        self.__CurCmd = TarballCommand.TarballCmd(attrs[ATTR_NAME], attrs[ATTR_TYPE])
	    else:
	        self.__CurCmd = TarballCommand.TarballCmd(attrs[ATTR_NAME])
	elif (name == B_ONFAIL):
	    if (attrs.has_key(ATTR_ARG)):
	        if (attrs.has_key(ATTR_TYPE)):
		    self.__CurCmd.AddCmdFail(attrs[ATTR_CMD], attrs[ATTR_ARG], attrs[ATTR_TYPE])
		else:
		    self.__CurCmd.AddCmdFail(attrs[ATTR_CMD], attrs[ATTR_ARG])
	    else:
                self.__CurCmd.AddCmdFail(attrs[ATTR_CMD], attrs[ATTR_ARG])
	elif (name == B_ONSUCCESS):
	    if (attrs.has_key(ATTR_ARG)):
	        if (attrs.has_key(ATTR_TYPE)):
		    self.__CurCmd.AddCmdSuccess(attrs[ATTR_CMD], attrs[ATTR_ARG], attrs[ATTR_TYPE])
		else:
		    self.__CurCmd.AddCmdSuccess(attrs[ATTR_CMD], attrs[ATTR_ARG])
	    else:
                self.__CurCmd.AddCmdSuccess(attrs[ATTR_CMD], attrs[ATTR_ARG])
        elif (name == B_VAR):
	    if self.__CurTarball:
                obj = self.__CurTarball
            elif self.__CurClass:
                obj = self.__CurClass

            if (attrs.has_key(ATTR_NAME) and attrs.has_key(ATTR_VALUE)):
                if attrs.has_key(ATTR_RESTRICTION):
                    obj.AddVar(attrs[ATTR_NAME], attrs[ATTR_VALUE], attrs[ATTR_RESTRICTION])
                else:
                    obj.AddVar(attrs[ATTR_NAME], attrs[ATTR_VALUE])

    # Action a faire pour les donnees d'une balise
    def CharacterData(self, data):
        """
        * Exceptions raised:
            - expat.ExpatError: Automatic raised by expat when there is
            syntax error.

        * Brief: Prototype expat, when it receives data in balise

        @param data: Data in between two balises
        @type data: string
	"""
        try:
	    name = self.__Balise.pop()
	except IndexError:
	    pass
	self.__Balise.append(name)
	self.__Data = data
	self.__Validator.ReceiveData(name, self.__Data)
	if (name == B_TEST_OS):
	    if (self.__CurTarball):
	        while ((data[:1] == " ") or (data[:1] == "\t") or (data[:1] == "\n")):
		    data = data[1:]
		if (data != ""):
	            self.__CurTarball.AddTestOs(data)

    # Action a faire a la fermeture d'une balise
    def EndElement(self, name):
        """
        * Exceptions raised:
            - expat.ExpatError: Automatic raised by expat when there is
            syntax error.

        * Exceptions intercepted:
            - HostClass.ExceptionWarn:when there is multiple defition or on
            generally when there is too attributs for one characters or when
            there is a lake of HotClass characters.

        * Brief: Prototype expat, when it exits in balise 'name'

        @param name: Exiting balise name
        @type name: string
	"""
        try:
	    self.__Balise.pop()
	except IndexError:
            pass
        self.__Validator.Exit(name, self.__Data)
	TmpAttr = self.__Attr.pop()
	if (name == B_CLASS):
	    if (self.__CurClass):
	        self.__ListClass.append(self.__CurClass)
	        self.__CurClass = None
	elif (name == B_CMD):
	    if (self.__CurCmd):
	        if (self.__CurTarball):
	            self.__CurTarball.AddCmd(self.__CurCmd)
	            self.__CurCmd = None
		elif (self.__CurClass):
	            self.__CurClass.AddCmd(self.__CurCmd)
	            self.__CurCmd = None

    def __init__(self, File):
        """
        * Exceptions raised: None

        * Exceptions intercepted:
            - expat.ExpatError:	We intercepted all exceptions raised
            by others object or others methos of this object.
            - IOError: When we have invalide file 'File'

        * Brief: The parser starting, we connect the expat parser to own parser
        and validator for XML Shema (doc/config/tarballs.xsd)

        @param File: File name to parse
        @type File: string
	"""
        self.__File		= File
	self.__ListClass 	= []
	self.__ListTarball	= []
        self.__CurClass 	= None
        self.__CurTarball	= None
	self.__CurCmd		= None
        self.__Balise 		= []
        self.__Attr 		= []
        self.__Data 		= ""
        self.__Parser = expat.ParserCreate()
        self.__Parser.StartElementHandler	= self.StartElement
        self.__Parser.EndElementHandler		= self.EndElement
        self.__Parser.CharacterDataHandler	= self.CharacterData
	try:
	    self.__Validator = TarballConfValidator(self.__Parser, self.__File)
            try:
	        self.__Parser.ParseFile(open(self.__File))
   	    except expat.ExpatError:
	        if (self.__Validator.GetError() == ""):
		    PrintMsg.ErrorXML(self.__File, self.__Parser.ErrorLineNumber, self.__Parser.ErrorColumnNumber, expat.ErrorString(self.__Parser.ErrorCode))
	        else:
		    PrintMsg.ErrorXML(self.__File, self.__Validator.GetErrorLine(), self.__Validator.GetErrorColumn(), self.__Validator.GetError())
	        self.__Validator.ResetError()
	except IOError, (errno, strerror):
            PrintMsg.VError(self.__File, strerror)

    ## DEBUG ##
    def __str__(self):
        Res = "StateParserTarballs:\n*===* List of Tarball Class *===*\n"

        for i in self.__ListClass:
	    Res += str(i) +"\n"
	Res += "*===============================*\n\n"
	Res += "*===* List of Tarball Object *===*\n"
        for i in self.__ListTarball:
	    Res += str(i) +"\n"
        return Res + "*================================*\n"

if __name__ == "__main__":
    i = ParserTarball(sys.argv[1])
    print str(i)
