#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: Dispatcher.py,v 1.1 2004/01/10 17:00:21 nes Exp $

"""
Used to compile and test all the tarball that are define in the XML
configuration.  If Daemon is on, you can ask the Dispatcher to add
other tarball, know some info and some ohter things.
"""

# FIXME: method names are not explicit enough and are mispelled.

import copy
import string
import re
import os, sys, time
import threading
from threading import Condition

import FinalTarball
import RemoteLoadAverage
import FinalHost
from common import *
from log import TestLog
from log.TestLogWriter import WriteXMLLog, XML_Header_write
import GetBestHost
import SetUsedHosts
from misc.thread_utilities import ThreadHandler

class Dispatcher(ThreadHandler):
    """
    Class used to make the dispatching of the tarballs on the hosts.
    """
    ## CONSTRUCTEUR
    def __init__(self, scheduler, FinalHostList,
                 FinalTarballList, host_free_cond):
        ThreadHandler.__init__(self, name = "dispatching")
        """
        @param FinalHostList : the list of FinalHost to dispatch.
        @param FinalTarball  : the list of Finaltarball to dispatch.
        @param cond          : condition that permit to manage threading
            module.
        @param cmd_stack     : an option on client mode.
        """
        self.__scheduler = scheduler
        self.__ListTarball = FinalTarballList
        self.__ListHost = FinalHostList
        self.__DicoTar = {}
        self._host_free_cond = host_free_cond
        ## creation of the dictionary of hosts.
        self.__Average = GetBestHost.GetBestHost(FinalHostList)
        ## creation of the disctionary of tarballs.
        self.__createDico()
        XML_Header_write()

    ## ACCESSORS

    def GetHostList(self):
        """ Get the list of the Host where found """
        return self.__ListHost

    def GetTarballList(self):
        """ Get the list of the tarball where defined to be execute"""
        return self.__ListTarball

    def GetAverage(self):
        return self.__Average

    def GetDicoTarball(self):
        """ accessor to the dicotionnary use to give tarball with an host """
        return self.__DicoTar


    ## METHODES

    def __createDico(self):
        """
        Method used to create a dictionnary where a key represents a tarball.
        Each key would be constitute by a list of ID classes. This list is
        making according to reference of variable.
        """
        for j in self.__ListTarball:
            spec = j.getTestableHost()
            list = self.__Average.getIDList(spec)
            if list == []:
                j_log = TestLog(j.getName(), "unknown", "none")
                j_log.setNotTested()
                j_log.setSummary("No compatible host.")
                self.__scheduler.addEvent(("test_result", j_log))
            else:
                self.__DicoTar[j] = list
        exit


    def Del_DicoID(self, FinalTarball, ID):
        """
        Method used to delete the ID classes on the DicoTar of this class
        This is necessary execute when a goodHost represented by an
        Id'Classes was found for this tarball.
        @param FinalTarball : the tarball which is completed on a host.
        @param Id           : the ID of FinallTarball to remove.
        """
        list = self.__DicoTar.get(FinalTarball)
        if list == None:
            return
        for j in list:
            if j == ID:
                list.remove(j)
                logger.info ("Tarball : %s is completed on %s"
                           % (FinalTarball.getName(), ID))
                if list == []:
                    logger.info ("(%s) No more check for tarball %s",
                                 self.getName(), FinalTarball.getName())
                    del self.__DicoTar[FinalTarball]
                return
        logger.info ("Error: no found ID on dicoTarball")

    def __del_DicoTar(self, FinalTarball):
        """
        Methode used to delete all Id in the Dictonnary that are present in the
        key : FinalTarball.
        @param FinallTarball : the key.
        """
        self.__DicoTar.pop(FinalTarball)

    def __add_DicoTar(self, FinalTarball, ID):
        """
        Method used to add an Id to a key in dico. the key is Finaltarball.
        @params FinalTarball : the finaltarball where is the key of the
        dictionnary.
        @param ID            : Id to add in the list of yhe dicotar
        represented by the key Finaltarball.

        """
        if self.__DicoTar.has_key(FinalTarball):
            s = self.__DicoTar.get(FinalTarball)
            s.append(ID)
        else:
            s = [ID]
        self.__DicoTar[j] = s
        logger.info ("Dico Create tarball : %s with success."
                   % FinalTarball.getName())

    def __del_Tarball_onDico(self, deletelist):
        """
        Method used to delete tarballs of a dictionary.
        @param deletelist : the list of tarballs to delete.
        """
        for i in deletelist:
            if self.__DicoTar.has_key(i):
                self.__DicoTar[i].pop()

    def Has_nexttarball(self):
        """
            Method is necesary because it permit to kno the next tarball to send
            if it's possible
            @return  : tuple (tarball, Host) if there is a tuple with all verifications.
                       tuple (None, None) If every hosts are occupied.
                       None if there are no more testable tarballs.
        """
        for tar in self.__DicoTar.keys():
            listeID = self.__DicoTar.get(tar)
            logger.debug ("(%s) Host list: %s.", self.getName(), str(listeID))
            for ID in listeID:
                try:
                    goodHost = self.__Average.getBestHost_ID(ID)
                except GetBestHost.AllBusyError,  e:
                    # Every hosts are busy for the moment.
                    logger.debug ("(%s) AllBusyError: %s", self.getName(), e)
                    logger.info ("(%s) All hosts are running tests. Waiting for host.",
                                 self.getName())
                    return None, None
                except GetBestHost.NoAvailableHostError:
                    # No available host and no hopes to get one.
                    logger.info ("(%s) No available host for ID `%s'", self.getName(), ID)
                    self.Del_DicoID(tar, ID)
                    tarball_log = TestLog(tar.getName(), ID, "none")
                    tarball_log.setNotTested()
                    tarball_log.setSummary("No available host.")
                    self.__scheduler.addEvent(("test_result", tarball_log))
                    # FIXME: this is a little tricky. Return values should be more explicit.
                    return None
                else:
                    return (tar, goodHost)
        # No more tarballs and ID to check.
        return None

    def run_(self):
        """
        Send a Tarball to Scheduler by the Way of the thread anyway,
        if any host are available, the dispathing module will be in
        Wait status waiting for a notify to shceduler (this is permit
        because the scheduler Know when an Host is going to be free
        and so must be used.
        """
        while self.__DicoTar != {}:
            logger.debug ("(%s) Remaining tests to perform: %s",
                          self.getName(), self.__DicoTar.values())
            # Throw away impossible tests.
            next_couple = self.Has_nexttarball()
            if next_couple:
                # There are still chances to run this test.
                tar, goodHost = next_couple
                if tar == None:
                    # No host for the moment, we have to wait.
                    self._host_free_cond.acquire()
                    logger.debug ("(%s) Waiting for host_free_cond", self.getName())
                    self._host_free_cond.wait()
                    self._host_free_cond.release()
                else:
                    # Let's start and send the tarball.
                    SetUsedHosts.set_host_in_use(goodHost, self.__ListHost, True)
                    self.Del_DicoID(tar, goodHost.getClassID())
                    self.__scheduler.addEvent(("new_test", (tar, goodHost)))
        self.__scheduler.addEvent(("terminated_task", self))
