#    Copyright (C) 2003, 2004, Maxime Biais <max@fr.fm>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: FinalHost.py,v 1.2 2004/01/10 18:55:09 nes Exp $

# FIXME: some functions have french names.
# FIXME: this files needs some cleanup before realease 0.2.
# Error handling seems weird and redundant.

import sys
import common
from common import logger

def get_chaine(ClassList, str):
    """
    Function used to get the hierachy string from ClassList.
    """
    res = []
    done = 0
    while not done:
        classid = get_classid(ClassList, str)
        res.insert(0, classid)
        str = ClassList[classid].getExtends()
        if (str == ""):
            done = 1
    return res

def get_classid(ClassList, str):
    """
    Function used to get the class ID from the class name in the ClassList.
    """
    j = 0
    for i in ClassList:
        if i.getName() == str:
            return j
        j += 1
    logger.error ("Class `%s' does not exist." % str)
    sys.exit(common.EXIT_XML)

def get_last_host_car(chaine, ClassList, str):
    """
    Function used to get the last host_car, following chaine in the ClassList.
    """
    for i in chaine:
        if ClassList[i].getHostCar() != None:
            return ClassList[i].getHostCar()
    logger.error ("Class `%s': undefined `host_car'." % str)
    sys.exit(common.EXIT_XML)

# FIXME: try to factorize this code
def get_last_connection_cmd(chaine, ClassList, str):
    """
    Function used to get the last connection_cmd, following chaine in the
    ClassList.
    """
    for i in chaine:
        if ClassList[i].getCmdConnec() != []:
            res = []
            for j in ClassList[i].getCmdConnec():
                res.append(j.GetCommand())
            return res
    logger.error ("Class `%s': undefined `connection_command'." % str)
    sys.exit(common.EXIT_XML)

def get_last_upload_cmd(chaine, ClassList, str):
    """
    Function used to get the last upload_cmd, following chaine in the ClassList
    """
    for i in chaine:
        if ClassList[i].getCmdUpload() != []:
            res = []
            for j in ClassList[i].getCmdUpload():
                res.append((j.GetLocation(), j.GetCommand()))
            return res
    logger.error ("Class `%s': undefined `upload_command'." % str)
    sys.exit(common.EXIT_XML)

def get_last_tmpdir(chaine, ClassList, str):
    """
    Function used to get the last tmpdir, following chaine in the ClassList
    """
    for i in chaine:
        if ClassList[i].getTmpDir() != "":
            return ClassList[i].getTmpDir()
    logger.error ("Class `%s': undefined `tmpdir'." % str)
    sys.exit(common.EXIT_XML)

def get_last_md5local(chaine, ClassList, str):
    """
    Function used to get the last tmpdir, following chaine in the ClassList.
    """
    for i in chaine:
        if ClassList[i].getMd5Local() != "":
            return ClassList[i].getMd5Local()
    return ""

def get_last_md5remote(chaine, ClassList, str):
    """
    Function used to get the last tmpdir, following chaine in the ClassList.
    """
    for i in chaine:
        if ClassList[i].getMd5Remote() != "":
            return ClassList[i].getMd5Remote()
    return ""

def host_car_conversion(HostCar):
    res = {
        "os"      : HostCar.getOs(),
        "archi"   : HostCar.getArchi(),
        "version" : HostCar.getVersion()
        }
    return res

def get_class_vars(path, ClassList):
    """
    Return a dictionary of vars defined a class hierarchy.
    """
    vars = {}
    for i in path:
       vars.update(ClassList[i].getVars())
    return vars

def transform(ClassList, HostList):
    """
    Function used to transform a ClassList and a HostList from the Host XML
    parser to a list of FinalHost.
    @param ClassList : the list of classes to use.
    @param HostList  : the list of hosts to use.
    @return          : the list of final hosts.
    @rtype           : list of FinalHost instanciations.
    """
    res = []
    env = {}
    cur_id = 0
    for h in range(0, len(HostList)):
        classname = HostList[h].getName()
        chaine = get_chaine(ClassList, classname)
        env = {}
        for i in chaine:
            cur_env = ClassList[i].getEnv()
            for j in cur_env:
                if env.has_key(j):
                    env.pop(j)
                env[j] = cur_env[j].GetValue()
        chaine.reverse()
        for i in HostList[h].getHosts():
            tmp = FinalHost()
            tmp.vars = get_class_vars(chaine, ClassList)
            tmp.name = i
            tmp.env = env
            tmp.host_car = host_car_conversion(get_last_host_car(chaine, ClassList, classname))
            tmp.connection_cmd = get_last_connection_cmd(chaine, ClassList, classname)
            tmp.upload_cmd = get_last_upload_cmd(chaine, ClassList, classname)
            tmp.tmpdir = get_last_tmpdir(chaine, ClassList, classname)
            tmp.md5local = get_last_md5local(chaine, ClassList, classname)
            tmp.md5remote = get_last_md5remote(chaine, ClassList, classname)
            tmp.classid = classname
            tmp.hostid = cur_id
            cur_id += 1
            res.append(tmp)
    return res

class FinalHost:
    """
    Class representing a host, with his name and all its caracteristics.
    """
    def __init__(self, name = "none", host_car = None, connection_cmd = None,
                 upload_cmd = None, env = None, tmpdir = "none",
                 classid = "none", used = False, md5local = "",
                 md5remote = "", vars = None):
        self.name             = name
        self.host_car         = host_car or {}
        self.connection_cmd   = connection_cmd or []
        self.upload_cmd       = upload_cmd or []
        self.env              = env or {}
        self.vars             = vars or {}
        self.tmpdir           = tmpdir
        self.md5local         = md5local
        self.md5remote        = md5remote
        self.classid          = classid
        self.used             = used
        self.hostid           = -1
        self.loadaverage      = -1


    def getName(self):
        """accessor"""
        return self.name

    def getHostCar(self):
        """accessor"""
        return self.host_car

    def getConnectionCmd(self):
        """accessor"""
        return self.connection_cmd

    def getUploadCmd(self):
        """accessor"""
        return self.upload_cmd

    def getEnv(self):
        """accessor"""
        return self.env

    def getVars(self):
        return self.vars

    def getTmpDir(self):
        """accessor"""
        return self.tmpdir

    def getClassID(self):
        """accessor"""
        return self.classid

    def getHostID(self):
        """accessor"""
        return self.hostid

    def getUsed(self):
        """accessor"""
        return self.used

    def setUsed(self, value):
        """accessor"""
        self.used = value

    def getLoadAverage(self):
        """accessor"""
        return self.loadaverage

    def setLoadAverage(self, value):
        try:
            self.loadaverage = float(value)
        except ValueError:
            print "vampire: WARNING: load average must be a float"
            self.loadaverage = 0


    def __str__(self):
        res  = "Hostname <%s>  ID = %d\n" % (self.name, self.hostid)
        res += "   ClassID               : " + self.classid + "\n"
        res += "   Host's caracteristics : " + str(self.host_car) + "\n"
        res += "   Environment variable  : " + str(self.env) + "\n"
        res += "   Connection Commands   : " + str(self.connection_cmd) + "\n"
        res += "   Upload Commands       : " + str(self.upload_cmd) + "\n"
        res += "   Tmp Directory         : " + str(self.tmpdir) + "\n"
        res += "   Used                  : " + str(self.used) + "\n"
        res += "   local md5 command     : " + self.md5local + "\n"
        res += "   remote md5 command    : " + self.md5remote + "\n"
        return res

def main():
    """Testing purpose"""
    import parserXMLconf, os, misc
    common.init_cmdline_options()
    hosts = parserXMLconf.ParserConf(os.path.expanduser(
        common.options.host_conf_file))
    hostClasses = hosts.GetListClass()
    hosts = hosts.GetListHost()
    res = transform(hostClasses, hosts)
    for i in res:
        print "----------"
        print i

if __name__ == "__main__":
    main()
