#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: FinalTarball.py,v 1.2 2004/01/10 18:55:09 nes Exp $

"""
Contains a function to transform an TarballList and a ClassList to a list of
FinalTarball. Also implements FinalTarball class.
"""

# FIXME: FACTORIZE THE CODE (it's awful)!!!
# FIXME: this file needs some cleanup before release 0.2.

from os.path import abspath, basename
import re

import common
from common import logger

def get_path(ClassList, str):
    """
    Get the hierachy list from ClassList
    For example::

             default(0)
                |
            ___________
           |          |
       debian(1)    freebsd(2)
           |
       debian2(3)

    Will return [3, 1, 0] if str is debian2
    @param str: The classname to look for parents
    @type str: string
    @return: the list conaining the path to its oldest parent
    """
    res = []
    done = 0
    while not done:
        classid = get_classid(ClassList, str)
        res.insert(0, classid)
        str = ClassList[classid].getExtends()
        if (str == ""):
            done = 1
    res.reverse()
    return res

def get_classid(ClassList, str):
    """
    get the class ID from the classname str in the ClassList
    @param str: the classname
    @type str: string
    @return: the class ID
    """
    j = 0
    for i in ClassList:
        if i.getName() == str:
            return j
        j += 1
    logger.error ("Class `%s' does not exist." % str)
    sys.exit(common.EXIT_XML)

def get_last_build_cmd(path, ClassList, str):
    """
    get the last build_cmd, following path in the ClassList
    @param path: path to follow
    @type path: string
    @param ClassList: ClassList to search in
    @type ClassList: ClassList
    @param str: classname
    @type str: string
    """
    for i in path:
        if ClassList[i].getListCmd() != []:
            res = []
            for j in ClassList[i].getListCmd():
                res.append(j)
            return res
    logger.error ("Class `%s': undefined `build_cmd'." % str)
    sys.exit(common.EXIT_XML)

def get_class_vars(path, ClassList):
    """
    Return a dictionary of vars defined a class hierarchy.
    """
    vars = {}
    for i in path:
       vars.update(ClassList[i].getVar())
    return vars

# FIXME: transform and transform files should be factorized!
def transform(ClassList, TarballList):
    """
    Transform The ClassList and TarballList to a FinalHost list
    @param TarballList: tarball list
    @type TarballList: TarballList
    @param ClassList: ClassList to search in
    @type ClassList: ClassList
    @return; the list of FinalTarball
    """
    res = []
    vars = {}
    for h in range(0, len(TarballList)):
        classname = TarballList[h].getClass()
        path = get_path(ClassList, classname)
        tmp = FinalTarball()
        tmp.name = TarballList[h].getName()
        tmp.docstring = "FIXME: nico ?!"
        tmp.vars = get_class_vars(path, ClassList)
        tmp.vars.update(TarballList[h].getVar())
        tmp.build_cmd = get_last_build_cmd(path, ClassList, classname)
        tmp.testable_host = TarballList[h].getTestOS()
        res.append(tmp)
    return res

def transform_files(ClassList, FileList, Tarballclass, TestableHost):
    """
    Transform The ClassList, a Tarballclass and a FileList to a
    FinalTarball list
    @param Tarballclass: tarball
    @type Tarballclass: Tarball
    @param FileList: list of the files to add
    @type FileList: list
    @param ClassList: ClassList to search in
    @type ClassList: ClassList
    @return; the list of FinalTarball
    """
    res = []
    vars = {}
    for h in range(0, len(FileList)):
        classname = Tarballclass
        path = get_path(ClassList, classname)
        tmp = FinalTarball()
        tmp.testable_host = TestableHost
        tmp.name = FileList[h]
        tmp.docstring = "None"
        try:
            dirname = re.match("(.*)\.tar\.[bz2|gz]", basename(tmp.name)).group(1)
        except:
            logger.error ("Can't get the dirname from the given parameter \
(%s)" % basename(tmp.name))
            dirname = "error"
        tmp.vars = {
            "param_name" : [(tmp.name, None)],
            "localfile" : [(abspath(tmp.name), None)],
            "tarballname" : [(basename(tmp.name), None)],
            "dirname" : [(dirname, None)]
            }
        tmp.vars.update(get_class_vars(path, ClassList))
        tmp.build_cmd = get_last_build_cmd(path, ClassList, classname)
        res.append(tmp)
    return res

class FinalTarball:
    """Class representing a tarball and its caracteristics"""
    def __init__(self, name = "none", docstring = "none", vars = None,
                 build_cmd = None, testable_host = ".*", md5 = "none"):
        self.name             = name
        self.docstring        = docstring
        self.vars             = vars or {}
        self.build_cmd        = build_cmd or []
        self.testable_host    = testable_host
        self.md5              = md5

    def getName(self):
        """accessor"""
        return self.name

    def getTarballName(self):
        return basename(self.name)

    def getDocString(self):
        """accessor"""
        return self.docstring

    def getVars(self):
        """accessor"""
        return self.vars

    def getBuildCmd(self):
        """accessor"""
        return self.build_cmd

    def getTestableHost(self):
        """accessor"""
        return self.testable_host

    def getMd5(self):
        """accessor"""
        return self.md5

    def __str__(self):
        """
        Pretty print the FinalTarball Class
        """
        res  = "Tarball                  : " + self.name + "\n"
        res += "   Docstring             : " + self.docstring + "\n"
        res += "   Environment variables : " + str(self.vars) + "\n"
        res += "   Build Commands        : " + str(self.build_cmd) + "\n"
        res += "   Testable Host         : " + self.testable_host + "\n"
        res += "   MD5                   : " + self.md5 + "\n"
        return res

def main():
    import parserXMLtarball, os, misc
    common.init_cmdline_options()
    tarbs = parserXMLtarball.ParserTarball(os.path.expanduser(
        common.options.tarball_conf_file))
    tarballClasses = tarbs.GetListClass()
    tarballs = tarbs.GetListTarball()
    res = transform(tarballClasses, tarballs)
    for i in res:
        print "----------"
        print i

if __name__ == "__main__":
    main()
