#    Copyright (C) 2003, 2004, Sebastien Henaff <henaff_s@epita.fr>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: GetBestHost.py,v 1.2 2004/01/10 18:55:09 nes Exp $


import string
import re
import FinalHost
import RemoteLoadAverage
import common
from common import *

# FIXME: this file needs some cleanup before release 0.2.
# Functions names are not explicit enough.
# Some code seems long and clumsy.

class NoAvailableHostError (Exception):
    def __init__(self, str):
          self.str = str

class AllBusyError (Exception):
   def __init__(self, str):
         self.str = str

class GetBestHost:
    """
    the class to determinate on which host a tarball will be tested.
    it depends on the load average of the host and its features.
    """
    def __init__(self, FinalHostList):
        """
        the constructor create a dictionnary keyed by the ID of the host, and
        containe a list of compatibles hosts.
        @param Name      : FinalHostList
        @param Type      : a list of  FinalHost
        """
        self.__DicoID = {}
        for i in FinalHostList:
              self.__add_DicoID(i)


    def __add_DicoID(self, FinalHost):
        """
        add a host to the dictionnary
        @param Name      : FinalHost
        @param Type      : FinalHost
        """
        ID = FinalHost.getClassID()
        if (self.__DicoID.has_key(ID)):
            self.__DicoID.get(ID).append(FinalHost)
        else:
            self.__DicoID[ID] = [FinalHost]


    def __del_DicoID(self, FinalHost):
        """
        del a host to the dictionnary
        @param Name      : FinalHost
        @param Type      : FinalHost
        """
        ID = FinalHost.getClassID()
        list = self.__DicoID.get(ID)
        for fh in list:
            if fh.getName() == FinalHost.getName():
                list.remove(fh)

    def del_DicoID(self, FinalHost):
        self.__del_DicoID(FinalHost)


    ## get the best host for only one ID##
    def getBestHost_ID(self, ID):
        #ID_list = self.getIDList(specifications)
        best = None
        AvailableHost = False
        if self.__DicoID.has_key(ID):
            #smallest = self.__DicoID.get(ID)[0]
            smallest = None
            for host in self.__DicoID.get(ID):
                if not(host.getUsed()):
                    if host.getLoadAverage() == -1:
                        try:
                              la = RemoteLoadAverage.RemoteLoadAverage(host)
                        except  RemoteLoadAverage.RemoteLoadAverageError, e:
                              pass
                        else:
                        ## gerer l'exception ..... d'echec
                              if la.getLoadAverage() != None:
                                    host.setLoadAverage(la.getLoadAverage())
                                    if host.getLoadAverage() <= common.options.load_average:
                                          return host
                                    if smallest != None:
                                          if host.getLoadAverage() < smallest.getLoadAverage():
                                                smallest = host
                                    else:
                                        smallest = host
		    else:
                        if host.getLoadAverage() <= common.options.load_average:
               		   return host
                           if smallest != None:
                    	        if host.getLoadAverage() < smallest.getLoadAverage():
                               	   smallest = host
                           else:
                               smallest = host
                else:
                      AvailableHost = True
            if (AvailableHost == False) & (smallest == None):
                  raise NoAvailableHostError("No Host is responding for this ID : %s " % (ID))
            if smallest != None:
                logger.info ("WARNING: there is no host with a load average lower than %s, so the host <%s> with a load average of %s has been selectionned"
                           % (common.options.load_average,
                              smallest.getName(),
                              smallest.getLoadAverage()))
                return smallest
        raise AllBusyError("No Host Free for the moment on this Id class")
        return None


        #logger.info ("No Host free for the moment")
        #raise RemoteLoadAverageError ("No Host free for the moment")


    ## get the id for add in tarball dico's
    def getIDList(self, specifications):
        """
        create the list of compatible ID in order to associate a tarball to a host.
        @param specifications : an regular expression
        @return : a list of ID which each is compatible with the specifications
        """
        ID_list = []
        for iter_id in self.__DicoID.iterkeys():
              if self.__DicoID.has_key(iter_id) and \
                     len (self.__DicoID.get(iter_id)) != 0:
                elt = self.__DicoID.get(iter_id)[0]
                spl = specifications.split(",")
                for iter_spec in spl:
                    found = False
                    #try:
                    #    pattern = iter_spec[int(iter_spec.index("=") + 1):]
                    #except ValueError:
                    #    pattern = iter_spec
                    pattern = specifications
                    for iter_env in elt.getEnv().iterkeys():
                        exp = iter_env + '=' + elt.getEnv()[iter_env]
                        try:
                            m = re.search(pattern, exp)
                            if m != None:
                                found = True
                                break
                        except re.error:
                            print "vampire: WARNING: the regexp %s is invalid" %pattern

                    if not(found):
                        for iter_car in elt.getHostCar().iterkeys():
                            exp = iter_car + '=' + elt.getHostCar()[iter_car]
                            try:
                                m = re.search(pattern, exp)
                                if m != None:
                                    found = True
                                    break
                            except re.error:
                                print "vampire: WARNING: the regexp %s is invalid" %pattern

                    if not(found):
                        break

                if found:
                    ID_list.append(iter_id)

        if len(ID_list) == 0:
            logger.info ("There is no Host compatible for all the specifications")
        return ID_list


if __name__ == "__main__":
    """Testing purpose"""
    import os
    import common, parserXMLconf, parserXMLtarball, FinalHost, FinalTarball
    common.init_cmdline_options()
    hosts = parserXMLconf.ParserConf(os.path.expanduser( \
        common.CMDLINE_OPTIONS["host-file"]))
    tarballs = parserXMLtarball.ParserTarball(os.path.expanduser( \
        common.CMDLINE_OPTIONS["tarball-file"]))
    FinalHostList = FinalHost.transform(hosts.GetListClass(),
                                        hosts.GetListHost())
    FinalTarballList = FinalTarball.transform(tarballs.GetListClass(),
                                              tarballs.GetListTarball())

    dico = GetBestHost(FinalHostList)
    #ID_List =  dico.getIDList("CXX=gcc,archi=.*")
    ID_List =  dico.getIDList(".*")
    print ID_List
    besthostlist = dico.getBestHost_List(ID_List)
    print besthostlist

    #besthost = dico.getBestHost_ID("gentoo")
    #print "Best host for gentoo", besthost
    #besthost = dico.getBestHost_ID("debian")
    #print "Best host for debian", besthost
