#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: RemoteAccess.py,v 1.2 2004/01/11 21:23:27 nes Exp $

"""

"""

import os, signal, sys, time, re
from misc import pexpect3
import common
from common import *

# String used to delimit the end of a command
ENDSTR="@E@N@D@"

class RemoteAccessError(Exception):
    """
    Exeption raised when there is an error in the RemoteAccess Module.
    """
    def __init__(self, str):
        self.__str = str
    def __str__(self):
        return self.__str

class RemoteAccess:
    """
    Remote access class for a remote shell command
    You must use a wrapper to use unusual command like telnet
    To write a wrapper : read src/wrappers/README
    """
    def __init__ (self, cmd, autoreconnect = 0):
        """
        Constructor:
        @param cmd: the command to run (it's often a wrapper call)
        @param autoreconnect: (not used) for compatibility with the
            old RemoteAccess module
        """
        self.__cmd   = cmd
        self.__autoreconnect = autoreconnect
        self.__reconnections = 0
        self.__isopen = 0

    def connect (self):
        """
        Used to connect on a remote host. Only run the command and
        check if the the command worked
        """
        logger.info ("Connecting (%s)" % self.__cmd)
        # Spawn the client and check
        try:
            self.__child = pexpect3.spawn(self.__cmd, timeout = 3600)
            self.__child.setecho(False)
        except pexpect3.ExceptionPexpect, e:
            self.__isopen = 0
            raise RemoteAccessError(str(e))

        # self.__child.setlog(sys.stdout)
        self.__isopen = 1

        # Test the connection and flush the banner
        i = 0
        for i in range(0, 6):
            self.__child.sendline("echo test")
            self.__child.sendline("echo %s" % ENDSTR)
            try:
                ret = self.__child.read_until(ENDSTR, 5)
                if not self.__child.isalive():
                    raise RemoteAccessError("Can't establish connection")
            except pexpect3.TIMEOUT:
                raise RemoteAccessError("Connection timeouted.")
        logger.info ("Connection established (%s)" % self.__cmd)

    def send_command2 (self, command):
        """
        Send a command to the child process
        @param command: the command to send
        @type command: string
        @return: stdout generated by the executed command
        """
        if (self.__isopen == 0):
            raise RemoteAccessError("Try to send a command on a closed connection")
        try:
            self.__child.sendline(command)
            self.__child.sendline("echo %s" % ENDSTR)
        except OSError, e:
            raise RemoteAccessError(e)
        try:
            ret = self.__child.read_until(ENDSTR)
        except pexpect3.TIMEOUT:
            raise RemoteAccessError("Timeout exceeded in read()")
        res = ""  
        for line in ret:
            res += line
        res = re.sub("\r\n", "\n", res)
        return res

    def send_command (self, command):
        """
        Send a command to the child process,
        @param command: the command to send
        @type command: string
        @return: a tuple: (stdout, exit_status)
        Don't use it if you don't need the exit status
        """
        if (self.__isopen == 0):
            raise RemoteAccessError("Try to send a command on a closed connection")
        try:
            self.__child.sendline(command)
            self.__child.sendline("echo \"%s:\"$?" % ENDSTR)
        except OSError, e:
            raise RemoteAccessError(e)
        try:
            ret, exit_st = self.__child.read_until_like(ENDSTR)
        except pexpect3.TIMEOUT:
            raise RemoteAccessError("Timeout exceeded in read()")
        res = ""
        exit_status = -1
        m = re.search("^%s:([-+]?\d+)$" % ENDSTR, exit_st)
        if m:
            exit_status = int(m.group(1))
        else:
            exit_status = -1
        for i in ret:
            res += i
        res = re.sub("\r\n", "\n", res)
        return res, exit_status

    def close (self):
        """Kill the child"""
        logger.info ("Try to close the connection")
        if (self.__isopen == 0):
            raise RemoteAccessError("You want to close a closed connection")
        # self.__child.close()
        try:
            self.__child.kill(signal.SIGINT)
            self.__child.kill(signal.SIGTERM)
            time.sleep(1)
            self.__child.kill(signal.SIGKILL)
        except:
            pass
        self.__isopen = 0
        logger.info ("Connection closed")


if __name__ == "__main__":
    """Tesing purposes"""
    init_cmdline_options()
    import sys
    try:
        ssh = RemoteAccess("ssh max@apoil sh", 3)
        ssh.connect()
    except:
        sys.exit(10)
    a = ssh.send_command2("echo 'test 1'")
    print "STDOUT :", a.strip()

    print ""
    a, e = ssh.send_command("ls")
    print "STDOUT :", a.strip()
    print "EXIT STATUS:", e
    print ""

    a = ssh.send_command2("/home/max/downloads/mopio/configure")
    print "STDOUT :", a.strip()
    ssh.close()
