#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: RemoteLoadAverage.py,v 1.2 2004/01/10 18:55:09 nes Exp $


import re

from RemoteAccess import RemoteAccess, RemoteAccessError
import FinalHost
from var_expansion import var_expand
from common import *

class RemoteLoadAverageError (Exception):
    def __init__(self, str):
        self.str = str

class RemoteLoadAverage:
    """
    the class to get the load average of a remote host (with the uptime unix command).
    """

    ## CONSTRUCTOR ##
    def __init__ (self, FinalHost):
        """
        the constructor get the load average of the FinalHost.
        @param Name      : FinalHost
        @param Type      : a FinalHost
        """
        self.__access = None
        try:
            self.__LoadAverage = self.__RemoteAverage(FinalHost)
        except RemoteLoadAverageError:
            self.__LoadAverage = None
            raise RemoteLoadAverageError ("Connection to host is impossible...")

    def __connect(self, FinalHost):
        """
        the method to connect the FinalHost.
        @param Name      : FinalHost
        @param Type      : a FinalHost
        """
        for i in FinalHost.getConnectionCmd():
            try:
                logger.info ("Making connection with <%s>" % FinalHost.getName())
                command = var_expand(None, FinalHost, i)
                logger.debug ("(%s) Trying connection with `%s'."
                              % (FinalHost.getName(), command))
                self.__access = RemoteAccess(command)
                self.__access.connect()
            except RemoteAccessError, e:
                self.__access = None
                logger.info ("(%s) Connection failed." % FinalHost.getName())
            if self.__access != None:
                return
        logger.error ("(%s) No connection command was successful."
                      % FinalHost.getName())
        raise RemoteLoadAverageError ("(%s) No connection command was successful."
                                      % FinalHost.getName())


    def __UptimeRegexp(self, str):
        """
        a method to get with a regular expression on the uptime output,
        the value of the load average. This value is balance by 0.2 for
        the last minute, 0.3 for the five last minutes and 0.5 for the
        fifteen las minutes.
        @param Name      : str
        @param Type      : string
        @param Return    : float
        """
        uptime = re.findall('\d\.\d+', str)
        if len(uptime) < 3:
            return 10
        return (float(uptime[0]) * 0.2 +
                float(uptime[1]) * 0.3 +
                float(uptime[2]) * 0.5)

    def __RemoteAverage(self, FinalHost):
        """
        the method to connect to the FinalHost, send the uptime command,
        and return it.
        @param Name      : FinalHost
        @param Type      : a FinalHost
        @param Return    : float
        """
        try:
            self.__connect(FinalHost)
        except RemoteLoadAverageError, e:
            ##probleme
            raise RemoteLoadAverageError ("All the connection commands have\
 been tested, but vampire can't connect to the remote host <"
                                          + FinalHost.getName() + ">")
        else:
            a, e = self.__access.send_command("uptime")
            uptime = self.__UptimeRegexp(a.strip())
            self.__access.close()
            logger.info ("Load average for <%s> is %s"
                       % (FinalHost.getName(), str(uptime)))
            return uptime

    ## ACCESSOR ##
    def getLoadAverage (self):
        """
        the accessor to get the load average, once calculated.
        @param Return    : float
        """
        return self.__LoadAverage

if __name__ == "__main__":
    """Testing purpose"""
    import os
    import common, parserXMLconf, parserXMLtarball, FinalHost, FinalTarball
    common.init_cmdline_options()
    hosts = parserXMLconf.ParserConf(os.path.expanduser(
        common.options.host_conf_file))
    tarballs = parserXMLtarball.ParserTarball(os.path.expanduser(
        common.options.tarball_conf_file))
    FinalHostList = FinalHost.transform(hosts.GetListClass(),
                                        hosts.GetListHost())
    FinalTarballList = FinalTarball.transform(tarballs.GetListClass(),
                                              tarballs.GetListTarball())

    rla = RemoteLoadAverage(FinalHostList[0])
    print rla.getLoadAverage()
