#    Copyright (C) 2003, 2004, Maxime Biais <max@fr.fm>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: Scheduler.py,v 1.1 2004/01/10 17:00:21 nes Exp $


"""
Handle tasks and events.
"""

from Queue import Queue
import copy
import time
import threading
from threading import Condition
import thread

from RemoteTest import RemoteTest
from log.TestLog import TestLog
from log import TestLogWriter
import SetUsedHosts
from Dispatcher import Dispatcher
from common import logger

class Scheduler:
    def __init__(self, FinalHostList, FinalTarballList, cond):
        self.__new_event_cond = cond
        self.__host_free_cond = Condition()
        self.__final_host_list = FinalHostList
        self.__final_tarball_list = FinalTarballList
        self.__tasks = {}
        self.__event_queue = Queue()

        self.__event_handlers = {
            'test_result' : self.handleTestResult,
            'terminated_task' : self.handleTerminatedTask,
            'new_test' : self.handleNewTest
            }

    def addEvent(self, event):
        """
        Add a new result. `task' should be the originally created task object.
        """
        self.__event_queue.put_nowait(event)
        logger.debug ("(%s) Notifying %s.",
                      threading.currentThread().getName(), event[0])
        self.notifyNewEvent()
        logger.debug ("(%s) Notified.", threading.currentThread().getName())

    def notifyNewEvent(self):
        """
        Tell to the scheduler that a task has been terminated.
        Blocks until the scheduler to be in wait mode.
        """
        self.__new_event_cond.acquire()
        self.__new_event_cond.notifyAll()
        self.__new_event_cond.release()

    def registerNewTask(self, task):
        logger.debug ("(scheduler) Add task %s.", task)
        self.__tasks[task] = None

    def GetDispatcher(self):
        return self.__dispatcher

    def handleNewTest(self, event_type, data):
        tarball, host = data
        r = RemoteTest(self, tarball, host)
        self.registerNewTask(r)
        r.start()

    def handleTerminatedTask(self, event_type, task):
        logger.debug ("(scheduler) Remove task %s.", task)
        del self.__tasks[task]

    def handleTestResult(self, event_type, log):
        logger.debug ("(scheduler) Handle test results.")
        TestLogWriter.WriteXMLLog(log)

    def handleEvents(self):
        while not self.__event_queue.empty():
	    event_type, data = self.__event_queue.get_nowait()
	    self.__event_handlers[event_type](event_type, data)

    def Wait_BeforeReloading(self):
        logger.debug ("(scheduler) Waiting for an event.")
        self.__new_event_cond.wait()

        logger.debug ("(scheduler) Check the event stack.")
        self.handleEvents()
        
        logger.debug ("(scheduler) Check if there are remaining tasks.")
        return len(self.__tasks) != 0

    def run(self):
        """
        Handle task creation and destruction.
        """
        logger.debug ("(scheduler) Started.")
        self.__new_event_cond.acquire()
        self.__dispatcher = Dispatcher(self,
                                       self.__final_host_list,
                                       self.__final_tarball_list,
                                       self.__host_free_cond)
        self.registerNewTask(self.__dispatcher)
        self.__dispatcher.start()
        while 1:
            if self.Wait_BeforeReloading() == False:
                break
        self.__new_event_cond.release()
        logger.debug ("(scheduler) Finished.")
