#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: remote_upload.py,v 1.2 2004/01/12 18:12:40 nes Exp $

"""
Send files and check Md5sum between local and remote files.
"""

import commands
import string
from RemoteAccess import RemoteAccess, RemoteAccessError
from common import *

class RemoteUploadError(Exception):
    """
    Exeption used for remote_upload module.
    """
    def __init__(self, str):
        self.__str = str
    def __str__(self):
        return self.__str

def upload(up_cmd):
    """
    Upload a file on a remote host.

    @param up_cmd: the command used to upload the file.
    @type up_cmd: string
    """
    logger.info ("Upload: %s" % up_cmd)
    try:
        (status, output) = commands.getstatusoutput(up_cmd)
    except:
        raise RemoteUploadError("can't upload using '%s'" % up_cmd)
    if status != 0:
        raise RemoteUploadError(output)

def download(down_cmd, access):
    """
    Download a file from a remote host.

    @param down_cmd: the command used to download the file from the host.
    @param access: the RemoteAccess instance to use.

    @type down_cmd: string
    @type access: RemoteAccess
    """
    logger.info ("Download from host: '%s'" % down_cmd)
    try:
        (output, status) = access.send_command(down_cmd)
    except RemoteAccessError, e:
        logger.info ("Error sending command: %s" % e)
        raise RemoteAccessError(str(e))
    if status != 0:
        raise RemoteUploadError(output)

def checkmd5_2(md5local, md5remote, access):
    """
    Check if the distant file is good.

    It executes two command: one locale and one remote.

    @param md5local: the local Md5 check command.
    @param md5remote: the remote Md5 check command.
    @param access: the RemoteAccess instance to use.

    @type md5local: string
    @type md5remote: string
    @type access: RemoteAccess

    @return: If Md5 check is good returns True, else False.
    @rtype: boolean
    """
    logger.info ("Check md5 integrety with command: %s" % md5remote)
    try:
        (hostoutput, hoststatus) = access.send_command(md5remote)
    except RemoteAccessError, e:
        logger.info ("Error sending command: %s" % e)
        raise RemoteAccessError(str(e))
    if hoststatus != 0:
        raise RemoteUploadError(hostoutput)
    (localstatus, localoutput) = commands.getstatusoutput(md5local)
    if localstatus != 0:
        raise RemoteUploadError(localoutput)
    return hostoutput.strip() == localoutput

def checkmd5(md5, md5remote, access):
    """
    Check if the distant file is good.

    It only executes the remote check, the local result is given in parameter.

    @param md5       : the Md5 of the local file to check.
    @param md5remote : the remote Md5 check command.
    @param access    : the RemoteAccess instance to use.

    @type md5: string
    @type md5remote: string
    @type access: RemoteAccess

    @return          : If Md5 check is good returns True, else False.
    @rtype: boolean
    """
    logger.info ("Check md5 integrety with command: %s" % md5remote)
    try:
        (output, status) = access.send_command(md5remote)
    except RemoteAccessError, e:
        logger.info ("Error sending command: %s" % str(e))
        raise RemoteAccessError(str(e))
    if status != 0:
        raise RemoteUploadError(output)
    return output.strip() == md5

def main():
    try:
        upload("scp README ben@gouarec.dyndns.org:~/")
    except RemoteUploadError, e:
        print "Error uploading:", e
    else:
        access = RemoteAccess("ssh ben@gouarec.dyndns.org bash")
        access.connect()
        if checkmd5("8e0093d02317ec6fe61e5bd226988f70",
                    "md5sum README | cut -d \" \" -f1",
                    access):
            print "CHECK OK"
        else:
            print "CHECK FAILED"
        access.close()

if __name__ == "__main__":
    init_cmdline_options()
    main()
