#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: var_expansion.py,v 1.2 2004/01/10 18:55:09 nes Exp $

"""
export a var_expansion function used to replace variables names with their
values in any string
"""

import re
from common import *

def check_restriction(restriction, classname):
    if restriction:
        return re.search(restriction, classname)
    else:
        return True
    
def __create_var_dic(FinalTarball, FinalHost):
    """
    Create a dictionnary with all usefull variables hold in FinalTarball
    and FinalHost
    @param FinalTarball: FinalTarball Object
    @param FinalHost: FinalHost Object
    @return: a dictionary associating a variable and its value
    """
    # FIXME: theses variable should be set elsewhere.
    res = {
        "HOSTNAME" : FinalHost.getName(),
        "TMPDIR" : FinalHost.getTmpDir()
        }
    res.update(FinalHost.getVars())
    if FinalTarball:
        res['REMOTEFILE'] = FinalHost.getTmpDir() + "/" + FinalTarball.getTarballName()
        vars = FinalTarball.getVars()
        for name, varlist in vars.items():
            for value, restriction in varlist:
                if check_restriction(restriction, FinalHost.getClassID()):
                    res[name.upper()] = value
                    break
    return res

def var_expand(FinalTarball, FinalHost, string):
    """
    Replace variables names with its associated value

    @param FinalTarball: FinalTarball Object
    @param FinalHost: FinalHost Object
    @param string: string to replace in
    @return: the modified string
    @rtype: string
    """
    var_dic = __create_var_dic(FinalTarball, FinalHost)
    res = ""
    try:
        res = string % var_dic
    except KeyError, e:
        logger.error ("Variable: %s does not exist in any configuration file"
                    % e)
        res = "[Vampire] Variable: %s does not exist in any configuration file." % e
    return res

def main():
    """Testing purpose"""
    import os
    import common, parserXMLconf, parserXMLtarball, FinalHost, FinalTarball
    common.init_cmdline_options()
    hosts = parserXMLconf.ParserConf(os.path.expanduser(
        common.options.host_conf_file))
    tarballs = parserXMLtarball.ParserTarball(os.path.expanduser(
        common.options.tarball_conf_file))
    FinalHostList = FinalHost.transform(hosts.GetListClass(),
                                        hosts.GetListHost())
    FinalTarballList = FinalTarball.transform(tarballs.GetListClass(),
                                              tarballs.GetListTarball())

    print var_expand(FinalTarballList[0], FinalHostList[0],
                     FinalHostList[0].getConnectionCmd()[0])
    print var_expand(FinalTarballList[0], FinalHostList[0],
                     FinalHostList[0].getUploadCmd()[0][1])
    print var_expand(None, FinalHostList[0],
                     FinalHostList[0].getConnectionCmd()[0])

if __name__ == "__main__":
    main()
