#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: TestLogHTMLOutput.py,v 1.1 2004/01/11 16:24:52 nes Exp $

class TestLogHTMLOutput:
    def __init__(self, basename, logs):
        self.__basename = basename
        self.__logs = logs

    def get_filename(self, classname):
        return self.__basename + "-" + classname + ".html"

    def Write(self):
        stream = open(self.__basename + ".html", "w")
        self.write_main_page(stream)
        stream.close()

        for test in self.__logs:
            stream = open(self.get_filename(test['class']), "w")
            self.write_full_log(test, stream)
            stream.close()

    def write_main_page(self, stream):
        stream.write(
"""
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
 <head>
  <title>Vampire Results</title>
 </head>
 <body>
 <center><h2>Vampire results</h2></center>
 <br>
""")

        stream.write("""
 <table border="1">
  <tr>
   <th>Tarball</th>
   <th>Class</th>
   <th>Summary</th>
   <th>Host</th>
   <th>Date</th>
   <th>Time</th>
   <th>Result</th>
   <th>Link</th>
  </tr>
""")

        for test in self.__logs:
            test['link'] = self.get_filename(test['class'])
            vars = test.copy()
            if test['status'] == 'OK':
                vars['color_status'] = '<font color="00FF00">OK</font>'
            elif test['status'] == 'FAILED':
                vars['color_status'] = '<font color="FF0000">FAILED</font>'
            elif test['status'] == 'NOT TESTED':
                vars['color_status'] = '<font color="0000FF">NOT TESTED</font>'
            else:
                vars['color_status'] = test['status']
            stream.write(
"""
  <tr>
   <td>%(tarball)s</td>
   <td>%(class)s</td>
   <td>%(summary)s</td>
   <td>%(host)s</td>
   <td>%(date)s</td>
   <td>%(time)s</td>
   <td><b>%(color_status)s</b></td>
   <td><a href=%(link)s>Details</a></td>
  </tr>
""" % vars)
        stream.write("""
 </table>
""")

        stream.write("""
 </body>
</html>
""")

    def write_full_log(self, test, stream):
        stream.write(
"""
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
 <head>
  <title>Vampire Results</title>
 </head>
 <body>
 <center><h2>Vampire results for %(tarball)s on %(class)s</h2></center>
 <br>
""" % test)

        stream.write("""
<h3>Summary</h3>
 <br>
 <a name=%(link)s></a>
 <table border="1">
  <tr>
   <th>Tarball</th>
   <th>Class</th>
   <th>Summary</th>
   <th>Host</th>
   <th>Date</th>
   <th>Time</th>
   <th>Result</th>
  </tr>
   <td>%(tarball)s</td>
   <td>%(class)s</td>
   <td>%(summary)s</td>
   <td>%(host)s</td>
   <td>%(date)s</td>
   <td>%(time)s</td>
   <td><b>%(status)s</b></td>
  </tr>
 </table>
""" % test)

        if test.has_key('cmd'):
            stream.write("""
<br>
<h3>Full log</h3>
 <br>
 <table border="1">
""")

            for cmd in test['cmd']:
                vars = cmd.copy()
                if cmd['status_cmd'] == "OK":
                    vars['color_name'] = "<font color=\"00FF00\">%s</font>" % cmd['name_cmd']
                elif cmd['status_cmd'] == "FAILED":
                    vars['color_name'] = "<font color=\"FF0000\">%s</font>" % cmd['name_cmd']
                else:
                    vars['color_name'] = cmd['name_cmd']
                stream.write("""
  <tr>
   <td><b>%(color_name)s</b></td>
   <td><b>%(status_cmd)s</b></td>
  </tr>
  <tr>
   <td colspan=2><pre>%(output)s</pre></td>
  <tr>
""" % vars)
		for info in cmd['infos']:
		    if info == "":
		        continue
		    stream.write(
"""
  <tr>
   <td colspan=2><b><font size=-1>(info)</font></b></td>
  </tr>
  <tr>
   <td colspan=2><pre>%s</pre></td>
  </tr>
""" % info)
		    
            stream.write("""
</table>
""")

        stream.write("""
 </body>
</html>
""")
