#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: TestLogWriter.py,v 1.4 2004/01/11 16:24:52 nes Exp $

# FIXME: remove successive fwrite, use one multiline string
# FIXME: should be a class, initialized by the Dispatcher.

import sys
from os.path import basename
import string

import TestLog
import common
from common import *

# Used to escapes merely character data to produce a well-formed XML
from xml.sax import saxutils

def XML_Header_write():
    """
    print the header of the XML log file
    """
    try:
        f = open(common.options.log_file, 'w')
        f.write ("<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n")
        f.write ("<logs xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n")
        f.write ("\n")
        f.write ("</logs>\n")
        f.close ()
        logger.info ("XML Header written")
    except IOError:
        common.logger.error ("The output file %s is corupted" % common.options.log_file)

def __KeepPrintableChars(str):
    res = ""
    for c in str:
        if string.find(string.printable, c) != -1:
            res += c
    return res

def __XMLescape(txt):
    "Escape XML entities"
    txt = __KeepPrintableChars(txt)
    txt = saxutils.escape(txt)
    return txt

def WriteXMLLog(Tarball_to_log):
    """
    add the logs of a tarball at the end of the output file.
    @param Name      : the tarball to log
    @param Type      : a TestLog
    """
    try:
        f = open(common.options.log_file, 'r+')
        tmp = f.readline ()
        while (tmp != "</logs>\n" and tmp != ""):
            pos = f.tell()
            tmp = f.readline ()
        if tmp == "":
            raise IOError

        f.seek(pos)

        values = {
            'tarball_name' : basename(Tarball_to_log.getTarballName ()),
            'host_name' : Tarball_to_log.getHostName (),
            'date' : Tarball_to_log.getDate (),
            'time' : Tarball_to_log.getTime (),
            'class_name' : Tarball_to_log.getClassName (),
            'summary' : Tarball_to_log.getSummary (),
            'status' : Tarball_to_log.getStatus ()
            }
        f.write ("""
  <tests>
    <tarball>%(tarball_name)s</tarball>
    <class>%(class_name)s</class>
    <status>%(status)s</status>
    <summary>%(summary)s</summary>
    <host>%(host_name)s</host>
    <date>%(date)s</date>
    <time>%(time)s</time>
""" % values)
        for cmd in Tarball_to_log.getLogs ():
            f.write ("    <cmd>\n")
            f.write ("      <name_cmd>" + __XMLescape(cmd.getName ()) + "</name_cmd>\n")
            #f.write ("<type>" + cmd.getType () + "</type>\n")
            if cmd.getStatus () == 0:
                f.write ("      <status_cmd>" + "OK" + "</status_cmd>\n")
            else:
                f.write ("      <status_cmd>" + "FAILED" + "</status_cmd>\n")
            f.write ("      <output>\n")
            f.write (__XMLescape(cmd.getOutput ()) + "\n")
            f.write ("      </output>\n")
            if cmd.getListInfos () == []:
                f.write ("      <infos/>\n")
            else:
                f.write ("      <infos>\n")
                for i in cmd.getListInfos ():
                    f.write ("        <info>" +
                             __XMLescape(i.encode (sys.getfilesystemencoding(),
                                       'xmlcharrefreplace')) + "</info>\n")
                f.write ("      </infos>\n")
            f.write ("    </cmd>\n")
        f.write ("  </tests>\n")
        f.write ("\n")
        f.write ("</logs>\n")
        f.close ()
    except IOError:
        common.logger.error ("The output file %s is corupted" \
                         % common.options.log_file)
    logger.info ("Logs for <%s> on <%s> have been written" \
               % (Tarball_to_log.getTarballName (),
                  Tarball_to_log.getHostName ()))


if __name__ == "__main__":
    import os
    import common, parserXMLconf, parserXMLtarball, FinalHost, FinalTarball
    from threading import Condition
    common.init_cmdline_options()
    hosts = parserXMLconf.ParserConf(os.path.expanduser(
        common.options.host_conf_file))
    tarballs = parserXMLtarball.ParserTarball(os.path.expanduser(
        common.options.tarball_conf_file))
    FinalHostList = FinalHost.transform(hosts.GetListClass(),
                                        hosts.GetListHost())
    FinalTarballList = FinalTarball.transform(tarballs.GetListClass(),
                                              tarballs.GetListTarball())

    logtar = TestLog.TestLog(FinalTarballList[0], FinalHostList[0])
    XML_Header_write ()
    WriteXMLLog(logtar)
    WriteXMLLog(logtar)
