#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: parse_opt.py,v 1.4 2004/01/12 18:12:40 nes Exp $

import common
import version
from optparse import OptionParser

def parse_command_line(actions):
    """
    Parse vampire command line.
    Read OptionParser documentation to get more information.
    """
    
    usage = """vampire [options] action [arguments]

actions:
  check TARBALL [TARBALL, ...]
                        Check the given tarballs.
"""
    parser = OptionParser(usage, version=version.VERSION)
    parser.add_option("-v", "--verbose",
                      action="store_true", dest="verbose", default=False,
                      help="verbose mode.")

    parser.add_option("-d", "--debug",
                      action="store_true", dest="debug", default=False,
                      help="debug mode.")

    parser.add_option("-n", "--do-nothing",
                      action="store_true", dest="do_nothing", default=False,
                      help="simulation mode, do not actually do anything.")

    parser.add_option("-t", "--tarball-conf", dest="tarball_conf_file",
                      default="~/.vampire/tarballs.xml", metavar="FILE",
                      help="tarball configuration file (`$HOME/.vampire/tarballs.xml').")

    parser.add_option("-c", "--host-conf", dest="host_conf_file",
                      default="~/.vampire/hosts.xml", metavar="FILE",
                      help="hosts configuration file (`$HOME/.vampire/hosts.xml').")

    parser.add_option("-r", "--host-classes", dest="host_classes",
                      default=".*", metavar="REGEXP",
                      help="host classes to check. (`.*').")

    parser.add_option("-q", "--tarball-class", dest="tarball_class",
                      default="default", metavar="CLASS",
                      help="tarball class to use. (`default').")

    parser.add_option("-a", "--max-load-average", dest="load_average",
                      default="0.8", metavar="FLOAT", type="float",
                      help="maximal authorized load average. (`0.8').")

    parser.add_option("-l", "--log-file", dest="log_file",
                      default="vampire-results.xml", metavar="FILE",
                      help="log file to store vampire results. (`vampire-results.xml').")

    (options, args) = parser.parse_args()

    if len(args) < 1:
        parser.error("An action is required.")

    action = args[0]
    action_args = args[1:]

    if not actions.has_key(action):
        parser.error("Wrong action.")

    if (actions[action] == len(action_args)) or \
       (actions[action] == -1 and len(action_args) >= 1):
        return options, action, action_args
    else:
        parser.error("Wrong action parameters.")
