#!/usr/bin/env python
#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: vampire-print-logs.py,v 1.2 2004/01/12 18:12:40 nes Exp $

"""
Convert Vampire XML output to more readable formats.
"""

import sys
from optparse import OptionParser

import version
from log.TestLogParser import TestLogParser
from log.TestLogHTMLOutput import TestLogHTMLOutput
from log.TestLogTextSummaryOutput import TestLogTextSummaryOutput

usage = """vampire-print-logs [options] logfile"""
parser = OptionParser(usage, version=version.VERSION)
parser.add_option("-m", "--html",
                  default=None, dest="html_basename", metavar="BASENAME",
                  help="Multiple files HTML output.  Specify the basename of the files.")

(options, args) = parser.parse_args()
if len(args) == 0:
    parser.error ("A vampire log file is required.")

parser = TestLogParser()
logs = parser.parse(args[0])

if options.html_basename:
    writer = TestLogHTMLOutput(options.html_basename, logs)
else:
    fields = [ ('tarball', 20),
               ('class', 20),
               ('summary', 30),
               ('status',  10) ]
    writer = TestLogTextSummaryOutput (logs, fields, sys.stdout)

writer.Write()
