#!/usr/bin/env python

#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: vampire.py,v 1.30 2004/01/10 18:55:09 nes Exp $

"""
Vampire main class
"""

import os
import sys
import traceback
from threading import Condition, Lock

import common
from common import logger
from conf.host_conf_parser import HostConfParser
from conf.tarball_conf_parser import TarballConfParser
from core import FinalHost
from core import FinalTarball
from core.Scheduler import Scheduler
from misc.parse_opt import parse_command_line

class Vampire:
    """Vampire main class"""
    def __init__(self):
        """Constructor"""
        self._hosts = None
        self._tarballs = None
        self._final_hosts = None
        self._final_tarballs = None
        self._event_terminated_cond = Condition()

        self._action_handlers = { 'check': self._handle_check }

    def _parse_cmd_line(self):
        """Parse command line."""
        actions = { "check" : -1 }
        (common.options, common.action,
         common.action_args) = parse_command_line(actions)
        # Postprocess some options
        common.options.host_conf_file = \
            os.path.expanduser(common.options.host_conf_file)
        common.options.tarball_conf_file = \
            os.path.expanduser(common.options.tarball_conf_file)

    def _parse_config_files(self):
        """Parse configuration files"""
        self._parsed_hosts = HostConfParser(common.options.host_conf_file)
        self._parsed_tarballs = TarballConfParser(common.options.tarball_conf_file)

    def _analyze_parsed_data(self):
        """
        Transform parsed data to _final_tarballs and _final_hosts
        containing all the tarballs to be test and all the host liable
        to test tarball
        """
        # Transform parsed hosts into final hosts, by analysing their classes.
        self._final_hosts = \
            FinalHost.transform(self._parsed_hosts.GetListClass(),
                                self._parsed_hosts.GetListHost())

        # Transform parsed tarballs into final tarballs.
        self._final_tarballs = \
            FinalTarball.transform_files(self._parsed_tarballs.GetListClass(),
                                         common.action_args,
                                         common.options.tarball_class,
                                         common.options.host_classes)

    def _run_scheduler(self):
        """Run the scheduler"""
        self._scheduler = Scheduler(self._final_hosts,
                                    self._final_tarballs,
                                    self._event_terminated_cond)
        self._scheduler.run()

    def _handle_check(self):
        self._parse_config_files()
        self._analyze_parsed_data()
        self._run_scheduler()

    def run(self):
        """Run vampire"""
        self._parse_cmd_line()
        common.initialize_logger(sys.stdout)
        try:
            self._action_handlers[common.action]()
        except:
            logger.error ("(main) Unexpected exception caught.  Exiting.")
            for line in traceback.format_exception(*sys.exc_info()):
                logger.error (line.rstrip("\n"))

if __name__ == "__main__":
    """Vampire main"""
    vampire = Vampire()
    vampire.run()
