#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: test_remote_access.py,v 1.10 2004/01/11 21:24:18 nes Exp $

import sys, re

from core.RemoteAccess import RemoteAccess, RemoteAccessError
from checkprimitives import *

nok = n = 0

# FIXME: Refaire un module de test ou l'on peut 'facilement' rajouter
# des tests

def OK(str):
    global n, nok
    n += 1
    nok += 1
    print_line(n, str, getstr_res(0, 0, 0));

def FAILED(str):
    global n
    n += 1
    print_line(n, str, getstr_res(0, 1, 0));

def PRINT_ERROR(got, expected):
    print "---- BEGIN ERROR GOT : ----"
    print got
    print "---- EXPECTED        : ----"
    print expected
    print "---- END         ERROR ----"

def PRINT_EXIT_STATUS(got, expected):
    print "---- EXIT STATUS : %d - EXPECTED : %d ----" % (got, expected)

def test_ret(test, command, expected):
    try:
        ssh = RemoteAccess("sh")
        ssh.connect()
        a = ssh.send_command2(command)
    except RemoteAccessError:
        FAILED(test)
    else:
        if a == expected:
            OK(test)
        else:
            FAILED(test)
            PRINT_ERROR(a, expected)
        ssh.close()

def test_ret2(test, command, expected, e_expected):
    try:
        ssh = RemoteAccess("sh")
        ssh.connect()
        a, e = ssh.send_command(command)
    except RemoteAccessError:
        FAILED(test)
    else:
        if re.match(expected, a) and e == e_expected:
            OK(test)
        else:
            FAILED(test)
            PRINT_ERROR(a, expected)
            PRINT_EXIT_STATUS(e, e_expected)
        ssh.close()

def test(vampire, vampiresrc):
    print "===== TESTING REMOTE ACCESS MODULE"

    __test = "Connection error"
    try:
        ssh = RemoteAccess("vampire_a_poil_ou_en_string")
        ssh.connect()
    except RemoteAccessError:
        OK(__test)
    else:
        FAILED(__test)
        ssh.close()

    __test = "Connection altered"
    try:
        ssh = RemoteAccess("cat")
        ssh.connect()
    except RemoteAccessError:
        OK(__test)
    else:
        FAILED(__test)
        ssh.close()

    __test = "Connection using ssh on dsfzeoifhzeux"
    try:
        ssh = RemoteAccess("ssh dsfzeoifhzeux")
        ssh.connect()
    except RemoteAccessError:
        OK(__test)
    else:
        FAILED(__test)
        ssh.close()

    __test = "Connection using sh"
    try:
        ssh = RemoteAccess("sh")
        ssh.connect()
    except RemoteAccessError:
        FAILED(__test)
    else:
        OK(__test)
        ssh.close()

    test_ret("Sending echo", "echo test", "test\n")
    test_ret("Sending echo \\n", "echo test\n", "test\n")
    test_ret("Sending echo a\\nb", "echo -e \"a\nb\"", "a\nb\n")
    test_ret("Sending multilines command", \
             "for i in `echo 1; echo 2; echo 3`; do\necho $i\ndone", "1\n2\n3\n")
    test_ret2("Sending echo and test exit status", "echo salut les jeunes !",
              "salut les jeunes !\n", 0)
    test_ret2("Sending wrong command", "vampire_en_string", ".*", 127)
    test_ret2("Sending ls error", "ls --vampire_a_poil", ".*", 1)
    test_ret2("Sending multiline command get exit status",
              "for i in `echo 1; echo 2; echo 3`; do\necho $i\ndone",
              "1\n2\n3\n", 0)
    test_ret2("Close double sh", "sh", ".*", 0)

    print "===== RESULT       : %d / %d" % (nok, n)
    return (nok, n)
