#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: test_remote_upload.py,v 1.4 2004/01/12 18:12:40 nes Exp $

import os, re
import commands
from checkprimitives import *
from core.remote_upload import *
from core.RemoteAccess import RemoteAccessError

##########################################################################

nok = n = 0

def OK(str):
    global n, nok
    n += 1
    nok += 1
    print_line(n, str, getstr_res(0, 0, 0));

def FAILED(str):
    global n
    n += 1
    print_line(n, str, getstr_res(0, 1, 0));

def PRINT_ERROR(got, expected):
    print "---- BEGIN ERROR GOT : ----"
    print got
    print "---- EXPECTED        : ----"
    print expected
    print "---- END         ERROR ----"

def PRINT_EXIT_STATUS(got, expected):
    print "---- EXIT STATUS : %d - EXPECTED : %d ----" % (got, expected)

##########################################################################

def test(vampire, vampiresrc):
    print "===== TESTING REMOTE UPLOAD MODULE"

    try:
        os.mknod ("/tmp/rut")
    except OSError, arg:
        pass


    try:
        output = commands.getoutput ("md5sum /tmp/rut | cut -d \" \" -f1")
        print output
        access = RemoteAccess("ssh localhost bash")
        access.connect()
        checkmd5 (output, "md5sum_bad", access)
        FAILED ("bad Md5 check")
    except RemoteUploadError, e:
        OK ("bad Md5 check")
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))

    try:
        upload ("scp /tmp/rut localhost:/tmp/rut2")
    except RemoteUploadError, e:
        FAILED ("Error uploading: " + str(e))
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))
    else:
        OK ("upload")

    try:
        access = RemoteAccess("ssh localhost bash")
        access.connect()
        download ("scp localhost:/tmp/rut /tmp/rut3", access)
    except RemoteUploadError, e:
        FAILED ("Error downloading: " + str(e))
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))
    else:
        OK ("download")

    try:
        output = commands.getoutput ("md5sum /tmp/rut | cut -d \" \" -f1")
        if checkmd5 (output, "md5sum /tmp/rut2 | cut -d \" \" -f1",
                     "ssh localhost"):
            OK ("check md5")
        else:
            FAILED ("check md5")
    except RemoteUploadError, e:
        FAILED ("Error md5 checking: " + str(e))
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))

    try:
        if checkmd5_2 ("md5sum /tmp/rut | cut -d \" \" -f1",
                       "md5sum /tmp/rut2 | cut -d \" \" -f1",
                       "ssh localhost"):
            OK ("check md5 (2)")
        else:
            FAILED ("check md5 (2)")
    except RemoteUploadError, e:
        FAILED ("Error md5 checking: " + str(e))
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))

    try:
        upload ("scp /tmp/inexistant_file localhost:/tmp/rut2")
    except RemoteUploadError, e:
        OK ("upload error")
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))
    else:
        FAILED ("upload error")

    try:
        upload ("scp /tmp/rut inexistant_host:/tmp/rut2")
    except RemoteUploadError, e:
        OK ("upload error")
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))
    else:
        FAILED ("upload error")

    try:
        download ("scp localhost:/tmp/inexistant_file /tmp/rut3",
                  "ssh localhost")
    except RemoteUploadError, e:
        OK ("download error")
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))
    else:
        FAILED ("download error")

    try:
        download ("scp localhost:/tmp/rut /tmp/rut3", "ssh inexistant_host")
    except RemoteUploadError, e:
        FAILED ("Error uploading: " + str(e))
    except RemoteAccessError, e:
        OK ("connection error")
    else:
        FAILED ("download error")

    try:
        if checkmd5 ("invalid md5", "md5sum /tmp/rut2", "ssh localhost"):
            FAILED ("check invalid md5")
        else:
            OK ("check invalid md5")
    except RemoteUploadError, e:
        FAILED ("Error md5 checking: " + str(e))
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))

    try:
        output = commands.getoutput ("md5sum /tmp/rut | cut -d \" \" -f1")
        checkmd5 (output, "md5sum_bad /tmp/rut2", "ssh localhost")
        FAILED ("bad Md5 check")
    except RemoteUploadError, e:
        OK ("bad Md5 check")
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))

    try:
        if checkmd5_2 ("md5sum /tmp/rut", "md5sum /tmp/rut2", "ssh localhost"):
            FAILED ("inavid check of md5 (2)")
        else:
            OK ("invalid check of md5 (2)")
    except RemoteUploadError, e:
        FAILED ("Error md5 checking: " + str(e))
    except RemoteAccessError, e:
        FAILED ("Error sending command: %s" % str(e))


    print "===== RESULT       : %d / %d" % (nok, n)
    return (nok, n)
