#!/usr/bin/env python

#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: sshwrapper.py,v 1.7 2004/01/11 18:57:41 nes Exp $

"""
Module used to autorespond 'yes' when ssh want you to add the host in your
known_host
"""

import sys
import re
import pexpect

ENDSTR="@E@N@D@"

class TelnetWrapper:
    def __init__(self, host, login, passwd, shell, telnetbin = "ssh"):
        """
        @param host: IP or hostname to connect on
        @param login: usernmae / login
        @param passwd: password
        @param shell: (unused) for compatibility with other wrapper
        @param telnetbin: telnet binary (default "ssh")
        """
        self.__bin      = telnetbin
        self.__host     = host
        self.__login    = login
        self.__passwd   = passwd
        self.__shell    = shell

    def connect(self):
        """Connect to the telnet server and send the password"""
        self.__child = pexpect.spawn(self.__bin + " -l %s %s %s"
                                     % (self.__login, self.__host,
                                        self.__shell))
        try:
            ret = self.__child.expect(["(yes/no)", "password:"], 10)
            if ret == 0:
                self.__child.sendline("yes")
                self.__child.expect("password:")
            self.__child.sendline(self.__passwd)
        except:
            pass

    def loop(self):
        """Interactive loop, never return"""
        self.__child.interact()

def main():
    if len(sys.argv) == 5 or len(sys.argv) == 6:
        telnet = TelnetWrapper(*sys.argv[1:])
        telnet.connect()
        telnet.loop()
    else:
        print "usage:   %s hostname login password shell [telnetbin]" \
        % sys.argv[0]
        print "example: %s 192.168.1.245 max max bash /usr/bin/telnet" \
        % sys.argv[0]

if __name__ == "__main__":
    main()
