#!/usr/bin/env python

#    Copyright (C) 2003, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: telnetwrapper.py,v 1.10 2003/09/20 20:23:05 maxme Exp $

import sys
import re
import pexpect

ENDSTR="@E@N@D@"

class TelnetWrapper:
    def __init__(self, host, login, passwd, shell, telnetbin = "telnet"):
        """
        @param host: IP or hostname to connect on
        @param login: usernmae / login
        @param passwd: password
        @param shell: (unused) for compatibility with other wrapper
        @param telnetbin: telnet binary (default "telnet")
        """
        self.__bin      = telnetbin
        self.__host     = host
        self.__login    = login
        self.__passwd   = passwd
        self.__shell    = shell

    def connect(self):
        """Connect to the telnet server and send the password"""
        self.__child = pexpect.spawn(self.__bin + " -l %s %s"
                                     % (self.__login, self.__host))
        # Wait for "Password: " on stdout
        self.__child.expect('Password: ')
        # And then send the password
        self.__child.sendline(self.__passwd)


    def flush_banner(self):
        """Send a test command to flush the banner"""
        self.__child.sendline(self.__shell)
        self.__child.sendline("echo test")
        self.__child.sendline("echo %s" % ENDSTR)
        ret = self.__child.read_until(ENDSTR)
        res = "\r\n".join(ret)
        res = re.sub("\r\n", "\n", res)
        return res

    def loop(self):
        """Interactive loop, never return"""
        self.__child.interact()

def main():
    if len(sys.argv) == 5 or len(sys.argv) == 6:
        telnet = TelnetWrapper(*sys.argv[1:])
        telnet.connect()
        telnet.flush_banner()
        telnet.loop()
    else:
        print "usage:   %s hostname login password shell [telnetbin]" \
        % sys.argv[0]
        print "example: %s 192.168.1.245 max max bash /usr/bin/telnet" \
        % sys.argv[0]

if __name__ == "__main__":
    main()
